@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,10 +18,12 @@
 */
 #include <gdk/gdkkeysyms.h>
 #include "add.h"
-#include "charlist.h"
 #include "general.h"
 #include "mm_list.h"
+#include "misc/charuty.h"
 #include "misc/fileio.h"
+#include "misc/misc_charset.h"
+#include "misc/misc_mdi.h"
 
 
 /******************************************************************************
@@ -30,11 +32,12 @@
 *                                                                             *
 ******************************************************************************/
 static GtkWidget *dialog, *button9, *check, *entry0, *entry4, *spin;
-static GtkWidget *combo1, *combo2, *combo3, *combo5, *combo7, *combo8;
+static GtkWidget *combo1, *combo2, *combo3, *combo5, *combo7, *charset8;
 static GtkWidget *label0, *label1, *label2, *label3;
 static GtkWidget *label4, *label5, *label7, *label8;
 
 
+#if ! GTK_CHECK_VERSION(2,4,0)
 /* ja:参照でOKボタンが押された */
 static void
 add_dialog_clicked_browse_ok (GtkWidget *widget,
@@ -47,6 +50,7 @@ add_dialog_clicked_browse_ok (GtkWidget *widget,
                 (GTK_FILE_SELECTION (dialog_browse)), -1, NULL, NULL, NULL);
   gtk_widget_destroy (dialog_browse);
 }
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
 
 
 /* ja:参照ボタンが押された */
@@ -54,9 +58,38 @@ static void
 add_dialog_clicked_browse (GtkWidget *widget,
                            gpointer   user_data)
 {
-  gchar *file = NULL;
+  gchar *file;
   GtkWidget *dialog_browse;
 
+#if GTK_CHECK_VERSION(2,4,0)
+  dialog_browse = gtk_file_chooser_dialog_new (_("Browse"),
+                        GTK_WINDOW (window), GTK_FILE_CHOOSER_ACTION_OPEN,
+                        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                        NULL);
+  file = g_filename_from_utf8 (gtk_entry_get_text (GTK_ENTRY (entry0)),
+                                                        -1, NULL, NULL, NULL);
+  if (fileio_isfile (file))
+    {
+      gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog_browse), file);
+    }
+  else
+    {
+      gchar *dir, *path;
+
+      dir = g_path_get_dirname (file);
+      path = fileio_get_full_path (dir);
+      g_free (dir);
+      if (fileio_isfile (path))
+        gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog_browse),
+                                                                        path);
+      g_free (path);
+    }
+  g_free (file);
+  file = gtk_dialog_run (GTK_DIALOG (dialog_browse)) == GTK_RESPONSE_ACCEPT
+    ? gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog_browse)) : NULL;
+  gtk_widget_destroy (dialog_browse);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   dialog_browse = gtk_file_selection_new (_("Browse"));
   g_signal_connect (G_OBJECT (dialog_browse), "destroy",
                                             G_CALLBACK (gtk_main_quit), NULL);
@@ -66,16 +99,23 @@ add_dialog_clicked_browse (GtkWidget *widget,
                 (G_OBJECT (GTK_FILE_SELECTION (dialog_browse)->cancel_button),
        "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog_browse));
   g_object_set_data (G_OBJECT (dialog_browse), "user_data", &file);
-  gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog_browse),
-                g_filename_from_utf8 (gtk_entry_get_text (GTK_ENTRY (entry0)),
-                                                        -1, NULL, NULL, NULL));
+  file = g_filename_from_utf8 (gtk_entry_get_text (GTK_ENTRY (entry0)),
+                                                        -1, NULL, NULL, NULL);
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog_browse), file);
+  g_free (file);
+  file = NULL;
   gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (dialog_browse));
   gtk_widget_show (dialog_browse);
   gtk_grab_add (dialog_browse);
   gtk_main ();
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   if (file)
     {
-      gtk_entry_set_text (GTK_ENTRY (entry0), file);
+      gchar *utf8str;
+
+      utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+      gtk_entry_set_text (GTK_ENTRY (entry0), utf8str);
+      g_free (utf8str);
       g_free (file);
     }
 }
@@ -86,9 +126,8 @@ static void
 add_dialog_clicked_ok (GtkWidget   *widget,
                        MmaidWindow *mmaid)
 {
-  gchar *file, *tmp;
-  const gchar *title, *artist, *album;
-  const gchar *year, *comment, *genre, *charset;
+  gchar *file, *tmp, *charset;
+  const gchar *title, *artist, *album, *year, *comment, *genre;
   gint n, c;
   ID3Tag *id3tag;
 
@@ -101,15 +140,29 @@ add_dialog_clicked_ok (GtkWidget   *widget,
   if (!file)
     return;
   id3tag = g_malloc0 (sizeof (ID3Tag));
+#if GTK_CHECK_VERSION(2,4,0)
+  title   = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (combo1)->child));
+  artist  = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (combo2)->child));
+  album   = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (combo3)->child));
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   title   = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo1)->entry));
   artist  = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo2)->entry));
   album   = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo3)->entry));
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   year    = gtk_entry_get_text (GTK_ENTRY (entry4));
+#if GTK_CHECK_VERSION(2,4,0)
+  comment = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (combo5)->child));
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   comment = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo5)->entry));
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   id3tag->track = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check))
               ? gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin)) : -1;
+#if GTK_CHECK_VERSION(2,4,0)
+  genre   = gtk_entry_get_text (GTK_ENTRY (GTK_BIN (combo7)->child));
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   genre   = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo7)->entry));
-  charset = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo8)->entry));
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
+  charset = misc_charset_get_charset (MISC_CHARSET (charset8));
   if (file && file[0] != '\0')
     id3tag->file = g_strdup (file);
   if (title[0] != '\0')
@@ -124,17 +177,20 @@ add_dialog_clicked_ok (GtkWidget   *widget,
     id3tag->comment = g_strdup (comment);
   if (genre[0] != '\0')
     id3tag->genre = g_strdup (genre);
-  if (charset[0] != '\0')
-    id3tag->charset = g_strdup (charset);
+  if (misc_charset_is_valid (MISC_CHARSET (charset8)))
+    id3tag->charset = charset;
+  else
+    g_free (charset);
   mm_list_get_cursor (MM_LIST (mmaid->list), &n, &c);
   n = n >= 0 ? mm_list_insert (MM_LIST (mmaid->list), n, id3tag)
              : mm_list_append (MM_LIST (mmaid->list), id3tag);
   mm_list_set_cursor (MM_LIST (mmaid->list), n, c);
   mm_list_unselect_all (MM_LIST (mmaid->list));
   mm_list_select (MM_LIST (mmaid->list), n);
-  if (charset[0] != '\0')
+  if (misc_charset_is_valid (MISC_CHARSET (charset8)))
     charlist_renewal_all (charset);
-  mmaid_change_edit (mmaid, TRUE);
+  misc_mdi_set_edited (MISC_MDI (mdi), 
+                    misc_mdi_get_page_from_data (MISC_MDI (mdi), mmaid), TRUE);
   gtk_widget_destroy (dialog);
 }
 
@@ -153,18 +209,24 @@ add_dialog_activate (GtkWidget  *widget,
 #define add_dialog_toggled add_dialog_changed
 
 
+/* ja:文字符号化方式が変更された */
+#define add_dialog_edited add_dialog_changed
+
+
 /* ja:コンボボックス/エントリー/スピンボタンが変更された */
 static void
 add_dialog_changed (GtkWidget   *widget,
                     MmaidWindow *mmaid)
 {
-  const gchar *charset;
+  gboolean result;
+  gchar *charset;
 
-  charset = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo8)->entry));
-  gtk_widget_set_sensitive (button9,
-                    mm_list_is_valid_file (MM_LIST (mmaid->list), -1,
+  charset = misc_charset_get_charset (MISC_CHARSET (charset8));
+  result = mm_list_is_valid_file (MM_LIST (mmaid->list), -1,
                                     gtk_entry_get_text (GTK_ENTRY (entry0)))
-                    && (charset[0] == '\0' || charlist_is_valid (charset)));
+    && (charset[0] == '\0' || misc_charset_is_valid (MISC_CHARSET (charset8)));
+  g_free (charset);
+  gtk_widget_set_sensitive (button9, result);
 }
 
 
@@ -185,10 +247,9 @@ add_dialog_key_press (GtkWidget    *widget,
 void
 add_dialog (MmaidWindow *mmaid)
 {
-  gchar **charset;
   gint i, n, c, count;
   GList *glist1 = NULL, *glist2 = NULL, *glist3 = NULL;
-  GList *glist5 = NULL, *glist7 = NULL, *glist8 = NULL;
+  GList *glist5 = NULL, *glist7 = NULL;
   GtkWidget *button0, *button10, *table, *hbox, *vbox;
   const ID3Tag *id3tag;
 
@@ -238,18 +299,34 @@ add_dialog (MmaidWindow *mmaid)
     if (genre[0][i] && !g_list_find_custom (glist7, genre[0][i],
                                                         (GCompareFunc)strcmp))
       glist7 = g_list_append (glist7, g_strdup (genre[0][i]));
-  charset = g_strsplit (charset_list, ",", G_MAXINT);
-  for (i = 0; charset[i]; i++)
-    glist8 = g_list_insert_sorted (glist8, g_strdup (charset[i]),
-                                                        (GCompareFunc)strcmp);
-  g_strfreev (charset);
   /* ja:コンボボックス */
+#if GTK_CHECK_VERSION(2,4,0)
+  combo1 = gtk_combo_box_entry_new_text ();
+  combo2 = gtk_combo_box_entry_new_text ();
+  combo3 = gtk_combo_box_entry_new_text ();
+  combo5 = gtk_combo_box_entry_new_text ();
+  combo7 = gtk_combo_box_entry_new_text ();
+  for (i = g_list_length (glist1) - 1; i >= 0; i--)
+    gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo1),
+                                                g_list_nth_data (glist1, i));
+  for (i = g_list_length (glist2) - 1; i >= 0; i--)
+    gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo2),
+                                                g_list_nth_data (glist2, i));
+  for (i = g_list_length (glist3) - 1; i >= 0; i--)
+    gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo3),
+                                                g_list_nth_data (glist3, i));
+  for (i = g_list_length (glist5) - 1; i >= 0; i--)
+    gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo5),
+                                                g_list_nth_data (glist5, i));
+  for (i = g_list_length (glist7) - 1; i >= 0; i--)
+    gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo7),
+                                                g_list_nth_data (glist7, i));
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   combo1 = gtk_combo_new ();
   combo2 = gtk_combo_new ();
   combo3 = gtk_combo_new ();
   combo5 = gtk_combo_new ();
   combo7 = gtk_combo_new ();
-  combo8 = gtk_combo_new ();
   if (glist1)
     gtk_combo_set_popdown_strings (GTK_COMBO (combo1), glist1);
   if (glist2)
@@ -260,22 +337,51 @@ add_dialog (MmaidWindow *mmaid)
     gtk_combo_set_popdown_strings (GTK_COMBO (combo5), glist5);
   if (glist7)
     gtk_combo_set_popdown_strings (GTK_COMBO (combo7), glist7);
-  if (glist8)
-    gtk_combo_set_popdown_strings (GTK_COMBO (combo8), glist8);
   gtk_combo_set_value_in_list (GTK_COMBO (combo1), FALSE, FALSE);
   gtk_combo_set_value_in_list (GTK_COMBO (combo2), FALSE, FALSE);
   gtk_combo_set_value_in_list (GTK_COMBO (combo3), FALSE, FALSE);
   gtk_combo_set_value_in_list (GTK_COMBO (combo5), FALSE, FALSE);
   gtk_combo_set_value_in_list (GTK_COMBO (combo7), FALSE, FALSE);
-  gtk_combo_set_value_in_list (GTK_COMBO (combo8), FALSE, FALSE);
   gtk_combo_set_case_sensitive (GTK_COMBO (combo1), TRUE);
   gtk_combo_set_case_sensitive (GTK_COMBO (combo2), TRUE);
   gtk_combo_set_case_sensitive (GTK_COMBO (combo3), TRUE);
   gtk_combo_set_case_sensitive (GTK_COMBO (combo5), TRUE);
   gtk_combo_set_case_sensitive (GTK_COMBO (combo7), TRUE);
-  gtk_combo_set_case_sensitive (GTK_COMBO (combo8), TRUE);
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   mm_list_get_cursor (MM_LIST (mmaid->list), &n, &c);
   id3tag = n >= 0 ? mm_list_get_nth (MM_LIST (mmaid->list), n) : NULL;
+#if GTK_CHECK_VERSION(2,4,0)
+  gtk_entry_set_text (GTK_ENTRY (GTK_BIN (combo1)->child),
+                            id3tag && id3tag->title ? id3tag->title : "");
+  gtk_entry_set_text (GTK_ENTRY (GTK_BIN (combo2)->child),
+                            id3tag && id3tag->artist ? id3tag->artist : "");
+  gtk_entry_set_text (GTK_ENTRY (GTK_BIN (combo3)->child),
+                            id3tag && id3tag->album ? id3tag->album : "");
+  gtk_entry_set_text (GTK_ENTRY (GTK_BIN (combo5)->child),
+                            id3tag && id3tag->comment ? id3tag->comment : "");
+  gtk_entry_set_text (GTK_ENTRY (GTK_BIN (combo7)->child),
+                            id3tag && id3tag->genre ? id3tag->genre : "");
+  g_signal_connect (G_OBJECT (GTK_BIN (combo1)->child), "activate",
+                                    G_CALLBACK (add_dialog_activate), mmaid);
+  g_signal_connect (G_OBJECT (GTK_BIN (combo2)->child), "activate",
+                                    G_CALLBACK (add_dialog_activate), mmaid);
+  g_signal_connect (G_OBJECT (GTK_BIN (combo3)->child), "activate",
+                                    G_CALLBACK (add_dialog_activate), mmaid);
+  g_signal_connect (G_OBJECT (GTK_BIN (combo5)->child), "activate",
+                                    G_CALLBACK (add_dialog_activate), mmaid);
+  g_signal_connect (G_OBJECT (GTK_BIN (combo7)->child), "activate",
+                                    G_CALLBACK (add_dialog_activate), mmaid);
+  g_signal_connect (G_OBJECT (GTK_BIN (combo1)->child), "changed",
+                                    G_CALLBACK (add_dialog_changed), mmaid);
+  g_signal_connect (G_OBJECT (GTK_BIN (combo2)->child), "changed",
+                                    G_CALLBACK (add_dialog_changed), mmaid);
+  g_signal_connect (G_OBJECT (GTK_BIN (combo3)->child), "changed",
+                                    G_CALLBACK (add_dialog_changed), mmaid);
+  g_signal_connect (G_OBJECT (GTK_BIN (combo5)->child), "changed",
+                                    G_CALLBACK (add_dialog_changed), mmaid);
+  g_signal_connect (G_OBJECT (GTK_BIN (combo7)->child), "changed",
+                                    G_CALLBACK (add_dialog_changed), mmaid);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo1)->entry),
                             id3tag && id3tag->title ? id3tag->title : "");
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo2)->entry),
@@ -286,8 +392,6 @@ add_dialog (MmaidWindow *mmaid)
                             id3tag && id3tag->comment ? id3tag->comment : "");
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo7)->entry),
                             id3tag && id3tag->genre ? id3tag->genre : "");
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo8)->entry),
-                            id3tag && id3tag->charset ? id3tag->charset : "");
   g_signal_connect (G_OBJECT (GTK_COMBO (combo1)->entry), "activate",
                                     G_CALLBACK (add_dialog_activate), mmaid);
   g_signal_connect (G_OBJECT (GTK_COMBO (combo2)->entry), "activate",
@@ -298,8 +402,6 @@ add_dialog (MmaidWindow *mmaid)
                                     G_CALLBACK (add_dialog_activate), mmaid);
   g_signal_connect (G_OBJECT (GTK_COMBO (combo7)->entry), "activate",
                                     G_CALLBACK (add_dialog_activate), mmaid);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo8)->entry), "activate",
-                                    G_CALLBACK (add_dialog_activate), mmaid);
   g_signal_connect (G_OBJECT (GTK_COMBO (combo1)->entry), "changed",
                                     G_CALLBACK (add_dialog_changed), mmaid);
   g_signal_connect (G_OBJECT (GTK_COMBO (combo2)->entry), "changed",
@@ -310,8 +412,7 @@ add_dialog (MmaidWindow *mmaid)
                                     G_CALLBACK (add_dialog_changed), mmaid);
   g_signal_connect (G_OBJECT (GTK_COMBO (combo7)->entry), "changed",
                                     G_CALLBACK (add_dialog_changed), mmaid);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo8)->entry), "changed",
-                                    G_CALLBACK (add_dialog_changed), mmaid);
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   for (i = g_list_length (glist1) - 1; i >= 0; i--)
     g_free (g_list_nth_data (glist1, i));
   for (i = g_list_length (glist2) - 1; i >= 0; i--)
@@ -322,14 +423,20 @@ add_dialog (MmaidWindow *mmaid)
     g_free (g_list_nth_data (glist5, i));
   for (i = g_list_length (glist7) - 1; i >= 0; i--)
     g_free (g_list_nth_data (glist7, i));
-  for (i = g_list_length (glist8) - 1; i >= 0; i--)
-    g_free (g_list_nth_data (glist8, i));
   g_list_free (glist1);
   g_list_free (glist2);
   g_list_free (glist3);
   g_list_free (glist5);
   g_list_free (glist7);
-  g_list_free (glist8);
+  /* ja:文字符号化方式 */
+  charset8 = misc_charset_new ();
+  misc_charset_add_candidate (MISC_CHARSET (charset8), charset_list);
+  misc_charset_set_charset (MISC_CHARSET (charset8),
+                            id3tag && id3tag->charset ? id3tag->charset : "");
+  g_signal_connect (G_OBJECT (charset8), "activate",
+                                    G_CALLBACK (add_dialog_activate), mmaid);
+  g_signal_connect (G_OBJECT (charset8), "edited",
+                                    G_CALLBACK (add_dialog_edited), mmaid);
   /* ja:エントリー */
   entry0 = gtk_entry_new ();
   entry4 = gtk_entry_new ();
@@ -373,14 +480,30 @@ add_dialog (MmaidWindow *mmaid)
   label5 = gtk_label_new_with_mnemonic (_("_Comment"));
   label7 = gtk_label_new_with_mnemonic (_("_Genre"));
   label8 = gtk_label_new_with_mnemonic (_("Character _Encoding"));
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label0), entry0);
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label1), GTK_COMBO (combo1)->entry);
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label2), GTK_COMBO (combo2)->entry);
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label3), GTK_COMBO (combo3)->entry);
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label4), entry4);
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label5), GTK_COMBO (combo5)->entry);
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label7), GTK_COMBO (combo7)->entry);
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label8), GTK_COMBO (combo8)->entry);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label0), entry0);
+#if GTK_CHECK_VERSION(2,4,0)
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label1), GTK_BIN (combo1)->child);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label2), GTK_BIN (combo2)->child);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label3), GTK_BIN (combo3)->child);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label1),
+                                                    GTK_COMBO (combo1)->entry);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label2),
+                                                    GTK_COMBO (combo2)->entry);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label3),
+                                                    GTK_COMBO (combo3)->entry);
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label4), entry4);
+#if GTK_CHECK_VERSION(2,4,0)
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label5), GTK_BIN (combo5)->child);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label7), GTK_BIN (combo7)->child);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label5),
+                                                    GTK_COMBO (combo5)->entry);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label7),
+                                                    GTK_COMBO (combo7)->entry);
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label8), charset8);
   /* ja:フレームとボックス */
   table = gtk_table_new (7, 4, FALSE);
   gtk_table_set_row_spacings (GTK_TABLE (table), SPACING);
@@ -432,7 +555,7 @@ add_dialog (MmaidWindow *mmaid)
   gtk_box_pack_end (GTK_BOX (hbox), label8, FALSE, FALSE, 0);
   gtk_table_attach (GTK_TABLE (table), hbox, 2, 3, 6, 7,
                                                     GTK_FILL, GTK_FILL, 0, 0);
-  gtk_table_attach_defaults (GTK_TABLE (table), combo8, 3, 4, 6, 7);
+  gtk_table_attach_defaults (GTK_TABLE (table), charset8, 3, 4, 6, 7);
   vbox = gtk_vbox_new (FALSE, SPACING);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
   gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
@@ -449,21 +572,37 @@ add_dialog (MmaidWindow *mmaid)
       case MM_LIST_FILE:
         gtk_widget_grab_focus (entry0);                    break;
       case MM_LIST_TITLE:
+#if GTK_CHECK_VERSION(2,4,0)
+        gtk_widget_grab_focus (GTK_BIN (combo1)->child);   break;
+      case MM_LIST_ARTIST:
+        gtk_widget_grab_focus (GTK_BIN (combo2)->child);   break;
+      case MM_LIST_ALBUM:
+        gtk_widget_grab_focus (GTK_BIN (combo3)->child);   break;
+#else /* not GTK_CHECK_VERSION(2,4,0) */
         gtk_widget_grab_focus (GTK_COMBO (combo1)->entry); break;
       case MM_LIST_ARTIST:
         gtk_widget_grab_focus (GTK_COMBO (combo2)->entry); break;
       case MM_LIST_ALBUM:
         gtk_widget_grab_focus (GTK_COMBO (combo3)->entry); break;
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
       case MM_LIST_YEAR:
         gtk_widget_grab_focus (entry4);                    break;
       case MM_LIST_COMMENT:
+#if GTK_CHECK_VERSION(2,4,0)
+        gtk_widget_grab_focus (GTK_BIN (combo5)->child);   break;
+      case MM_LIST_TRACK:
+        gtk_widget_grab_focus (check);                     break;
+      case MM_LIST_GENRE:
+        gtk_widget_grab_focus (GTK_BIN (combo7)->child);   break;
+#else /* not GTK_CHECK_VERSION(2,4,0) */
         gtk_widget_grab_focus (GTK_COMBO (combo5)->entry); break;
       case MM_LIST_TRACK:
         gtk_widget_grab_focus (check);                     break;
       case MM_LIST_GENRE:
         gtk_widget_grab_focus (GTK_COMBO (combo7)->entry); break;
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
       case MM_LIST_CHARSET:
-        gtk_widget_grab_focus (GTK_COMBO (combo8)->entry); break;
+        gtk_widget_grab_focus (charset8);                  break;
       default:
         gtk_widget_grab_focus (button10);
     }
