@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -165,6 +165,18 @@ command_charset (gpointer   callback_data,
                  GtkWidget *widget);
 
 
+void
+command_play (gpointer   callback_data,
+              guint      callback_action,
+              GtkWidget *widget);
+
+
+void
+command_stop (gpointer   callback_data,
+              guint      callback_action,
+              GtkWidget *widget);
+
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
