@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,10 +18,10 @@
 */
 #include <gdk/gdkkeysyms.h>
 #include "export.h"
-#include "charlist.h"
 #include "general.h"
 #include "mm_list.h"
 #include "misc/fileio.h"
+#include "misc/misc_charset.h"
 
 
 /******************************************************************************
@@ -29,7 +29,7 @@
 * ja:エクスポートダイアログ関数群                                             *
 *                                                                             *
 ******************************************************************************/
-static GtkWidget *dialog, *button0, *check, *combo, *label0, *spin;
+static GtkWidget *dialog, *button0, *charset, *check, *label0, *spin;
 
 
 /* ja:OKボタンが押された */
@@ -39,8 +39,7 @@ export_dialog_clicked (GtkWidget   *widget,
 {
   *expinfo = g_malloc0 (sizeof (ExportInfo));
   (*expinfo)->gnum = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin));
-  (*expinfo)->charset = g_strdup (gtk_entry_get_text
-                                    (GTK_ENTRY (GTK_COMBO (combo)->entry)));
+  (*expinfo)->charset = misc_charset_get_charset (MISC_CHARSET (charset));
   (*expinfo)->hold = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
   charlist_renewal_all ((*expinfo)->charset);
   gtk_widget_destroy (dialog);
@@ -57,42 +56,42 @@ export_dialog_activate (GtkWidget  *widget,
 }
 
 
-/* ja:コンボボックスが変更された */
+/* ja:文字符号化方式が変更された */
 static void
-export_dialog_changed_combo (GtkWidget *widget,
-                             gpointer   user_data)
+export_dialog_edited (GtkWidget *widget,
+                      gpointer   user_data)
 {
-  const gchar *charset;
   gint g;
 
   g = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin));
-  charset = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry));
   gtk_widget_set_sensitive (button0, 0 <= g && g <= 255
-                                            && charlist_is_valid (charset));
+                            && misc_charset_is_valid (MISC_CHARSET (charset)));
 }
 
 
 /* ja:スピンボタンが変更された */
 static void
-export_dialog_changed_spin (GtkWidget *widget,
-                            gpointer   user_data)
+export_dialog_changed (GtkWidget *widget,
+                       gpointer   user_data)
 {
   gchar *str = "";
-  const gchar *charset;
-  gint g, i;
+  gint g;
 
   g = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin));
   if (0 <= g && g <= 255)
-    for (i = 0; i < genres; i++)
-      if (genre[i][g])
-        {
-          str = genre[i][g];
-          break;
-        }
+    {
+      gint i;
+
+      for (i = 0; i < genres; i++)
+        if (genre[i][g])
+          {
+            str = genre[i][g];
+            break;
+          }
+    }
   gtk_label_set_text (GTK_LABEL (label0), str);
-  charset = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry));
   gtk_widget_set_sensitive (button0, 0 <= g && g <= 255
-                                            && charlist_is_valid (charset));
+                            && misc_charset_is_valid (MISC_CHARSET (charset)));
 }
 
 
@@ -113,9 +112,7 @@ export_dialog_key_press (GtkWidget    *widget,
 ExportInfo *
 export_dialog (const gchar *def_charset)
 {
-  gchar **charset;
   gint i;
-  GList *glist = NULL;
   GtkWidget *button1, *frame, *label1, *hbox, *vbox;
   ExportInfo *expinfo = NULL;
 
@@ -135,25 +132,14 @@ export_dialog (const gchar *def_charset)
                         G_CALLBACK (gtk_widget_destroy), G_OBJECT (dialog));
   GTK_WIDGET_SET_FLAGS (button0, GTK_CAN_DEFAULT);
   GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
-  /* ja:コンボボックス */
-  charset = g_strsplit (charset_list, ",", G_MAXINT);
-  for (i = 0; charset[i]; i++)
-    glist = g_list_insert_sorted (glist, g_strdup (charset[i]),
-                                                        (GCompareFunc)strcmp);
-  g_strfreev (charset);
-  combo = gtk_combo_new ();
-  if (glist)
-    gtk_combo_set_popdown_strings (GTK_COMBO (combo), glist);
-  gtk_combo_set_value_in_list (GTK_COMBO (combo), FALSE, FALSE);
-  gtk_combo_set_case_sensitive (GTK_COMBO (combo), TRUE);
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), def_charset);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo)->entry), "activate",
-                            G_CALLBACK (export_dialog_activate), &expinfo);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo)->entry), "changed",
-                            G_CALLBACK (export_dialog_changed_combo), NULL);
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
-    g_free (g_list_nth_data (glist, i));
-  g_list_free (glist);
+  /* ja:文字符号化方式 */
+  charset = misc_charset_new ();
+  misc_charset_add_candidate (MISC_CHARSET (charset), charset_list);
+  misc_charset_set_charset (MISC_CHARSET (charset), def_charset);
+  g_signal_connect (G_OBJECT (charset), "activate",
+                                G_CALLBACK (export_dialog_activate), &expinfo);
+  g_signal_connect (G_OBJECT (charset), "edited",
+                                G_CALLBACK (export_dialog_edited), NULL);
   /* ja:チェックボタン */
   check = gtk_check_button_new_with_mnemonic (_("_Hold existing tags"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), def_hold);
@@ -162,9 +148,9 @@ export_dialog (const gchar *def_charset)
                                         (def_gnum, 0, 255, 1, 5, 0)), 0, 0);
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spin), TRUE);
   g_signal_connect (G_OBJECT (spin), "activate",
-                            G_CALLBACK (export_dialog_activate), &expinfo);
+                                G_CALLBACK (export_dialog_activate), &expinfo);
   g_signal_connect (G_OBJECT (spin), "changed",
-                            G_CALLBACK (export_dialog_changed_spin), NULL);
+                                G_CALLBACK (export_dialog_changed), NULL);
   /* ja:フレーム */
   frame = gtk_frame_new (NULL);
   label1 = gtk_label_new_with_mnemonic (_("_Genre"));
@@ -176,7 +162,7 @@ export_dialog (const gchar *def_charset)
       break;
   label0 = gtk_label_new (i < genres ? genre[i][def_gnum] : "");
   label1 = gtk_label_new_with_mnemonic (_("Character _Encoding"));
-  gtk_label_set_mnemonic_widget (GTK_LABEL (label1), GTK_COMBO (combo)->entry);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label1), charset);
   /* ja:ボックス */
   vbox = gtk_vbox_new (FALSE, SPACING);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
@@ -188,7 +174,7 @@ export_dialog (const gchar *def_charset)
   gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), charset, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), check, FALSE, FALSE, 0);
   hbox = gtk_hbox_new (FALSE, SPACING);
