@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,202 +24,8 @@
 #include "sigmain.h"
 #include "misc/fileio.h"
 #include "misc/misc.h"
-
-
-/******************************************************************************
-*                                                                             *
-* ja:ファイル関数群                                                           *
-*                                                                             *
-******************************************************************************/
-/*  ja:ファイルの履歴に加える
-    file,ファイル名                                                         */
-void
-file_set_history (const gchar *file)
-{
-  gchar *label;
-  gint i, count;
-  GList *glist;
-  GtkWidget *menu_shell, *menu_item;
-
-  menu_shell = gtk_item_factory_get_widget (ifactory_menu, "<main>/File");
-  glist = gtk_container_children (GTK_CONTAINER (menu_shell));
-  count = g_list_length (glist);
-  for (i = 0; i < count - MENUFILE - 1; i++)
-    {
-      menu_item = g_list_nth_data (glist, i + MENUFILE - 1);
-      label = g_object_get_data (G_OBJECT (menu_item), "user_data");
-      if (!g_strfilecmp (label, file))
-        {
-          gtk_container_remove (GTK_CONTAINER (menu_shell), menu_item);
-          break;
-        }
-    }
-  g_list_free (glist);
-  if (i >= history)
-    {
-      glist = gtk_container_children (GTK_CONTAINER (menu_shell));
-      gtk_container_remove (GTK_CONTAINER (menu_shell),
-                        g_list_nth_data (glist, MENUFILE + history + 1 - 3));
-      g_list_free (glist);
-    }
-  if (count <= MENUFILE)
-    {
-      menu_item = gtk_menu_item_new ();
-      gtk_menu_insert (GTK_MENU (menu_shell), menu_item, MENUFILE - 2);
-        gtk_widget_show (menu_item);
-    }
-  label = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-  menu_item = gtk_menu_item_new_with_label (label);
-  g_free (label);
-  label = g_strdup (file);
-  g_signal_connect (G_OBJECT (menu_item), "activate",
-                            G_CALLBACK (signal_activate_menu_history), label);
-  g_signal_connect (G_OBJECT (menu_item), "destroy",
-                            G_CALLBACK (signal_destroy_menu_history), label);
-  g_object_set_data (G_OBJECT (menu_item), "user_data", label);
-  gtk_menu_insert (GTK_MENU (menu_shell), menu_item, MENUFILE - 1);
-  gtk_widget_show (menu_item);
-}
-
-
-/*  ja:ファイルを加える
-    file,加えようとするファイルのフルパス
-    same,フルパスに付属する数値
-     RET,加えようとするファイルのラベル                                     */
-gchar *
-file_add_edit (const gchar *file, gint *same)
-{
-  gchar *label, *utf8str;
-  const gchar *name0, *name1;
-  gint i;
-  GList *glist;
-  GtkWidget *child;
-  MmaidWindow *mmaid;
-
-  *same=-1;
-  /* ja:ファイル名を調べる */
-  glist = gtk_container_children (GTK_CONTAINER (notebook));
-  /* ja:同名同パスを調べる */
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
-    {
-      child = g_list_nth_data (glist, i);
-      mmaid = g_object_get_data (G_OBJECT (child), "user_data");
-      if (!g_strfilecmp (file, mmaid->file))
-        {
-          if (mmaid->same < 0)
-            {
-              mmaid->same = 0;
-              utf8str = g_filename_to_utf8 (mmaid->file, -1, NULL, NULL, NULL);
-              label = g_strdup_printf ("%s:%d", utf8str, mmaid->same);
-              g_free (utf8str);
-              gtk_label_set_text (GTK_LABEL (mmaid->label), label);
-              g_free (label);
-            }
-            if (*same <= mmaid->same)
-              *same = mmaid->same + 1;
-        }
-    }
-  if (*same >= 0)
-    {
-      /* ja:同名同パスがあるとき */
-      utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-      label = g_strdup_printf ("%s:%d", utf8str, *same);
-      g_free (utf8str);
-    }
-  else
-    {
-      /* ja:同名同パスはないとき */
-      /* ja:同名異パスを調べる */
-      name0 = g_basename (file);
-      for (i = g_list_length (glist) - 1; i >= 0; i--)
-        {
-          child = g_list_nth_data (glist, i);
-          mmaid = g_object_get_data (G_OBJECT (child), "user_data");
-          name1 = g_basename (mmaid->file);
-          if (!g_strfilecmp (name0, name1))
-            {
-              if (mmaid->same < 0)
-                {
-                  utf8str = g_filename_to_utf8 (mmaid->file, -1,
-                                                            NULL, NULL, NULL);
-                  gtk_label_set_text (GTK_LABEL (mmaid->label), utf8str);
-                  g_free (utf8str);
-                }
-              break;
-            }
-        }
-      label = g_filename_to_utf8 (i >= 0 ? file : name0, -1, NULL, NULL, NULL);
-    }
-  g_list_free (glist);
-  return label;
-}
-
-
-/*  ja:ファイルを削除する
-    file,削除しようとするファイルのフルパス                                 */
-void
-file_delete_edit (const gchar *file)
-{
-  gchar *utf8str;
-  const gchar *name0,*name1;
-  gint i,count=0;
-  GList *glist;
-  MmaidWindow *mmaid;
-  GtkWidget *child;
-
-  /* ja:ファイル名を調べる */
-  glist = gtk_container_children (GTK_CONTAINER (notebook));
-  /* ja:同名異パスを調べる */
-  name0 = g_basename (file);
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
-    {
-      mmaid = g_object_get_data (g_list_nth_data (glist, i), "user_data");
-      name1 = g_basename (mmaid->file);
-      if (!g_strfilecmp (name0, name1))
-        count++;
-    }
-  if (count <= 2)
-    {
-      /* ja:同名異パスのファイルを通常表示に変更する */
-      for (i = g_list_length (glist) - 1; i >= 0; i--)
-        {
-          child = g_list_nth_data (glist, i);
-          mmaid = g_object_get_data (G_OBJECT (child), "user_data");
-          name1 = g_basename (mmaid->file);
-          if (!g_strfilecmp (name0, name1))
-            {
-              utf8str = g_filename_to_utf8 (name1, -1, NULL, NULL, NULL);
-              gtk_label_set_text (GTK_LABEL (mmaid->label), utf8str);
-              g_free (utf8str);
-            }
-        }
-    }
-  else
-    {
-      /* ja:同名同パスを調べる */
-      count = 0;
-      for (i = g_list_length (glist) - 1; i >= 0; i--)
-        {
-          mmaid = g_object_get_data (g_list_nth_data (glist, i), "user_data");
-          if (!g_strfilecmp (file, mmaid->file))
-            count++;
-        }
-      if (count <= 2)/* ja:同名同パスのファイルをフルパス表示に変更する */
-        for (i = g_list_length (glist) - 1; i >= 0; i--)
-          {
-            child = g_list_nth_data (glist, i);
-            mmaid = g_object_get_data (G_OBJECT (child), "user_data");
-            if (!g_strfilecmp (file, mmaid->file))
-              {
-                mmaid->same = -1;
-                utf8str = g_filename_to_utf8 (mmaid->file, -1,
-                                                            NULL, NULL, NULL);
-                gtk_label_set_text (GTK_LABEL (mmaid->label), utf8str);
-                g_free (utf8str);
-              }
-          }
-    }
-}
+#include "misc/misc_history.h"
+#include "misc/misc_mdi.h"
 
 
 /******************************************************************************
@@ -269,19 +75,20 @@ static const gchar irivdb_header1[] = {
         err,TRUE:エラーなし,FALSE:強行
         RET,変換されたファイル名,NULL:エラー                                */
 static gchar *
-file_path_convert (const gchar *file,
-                   const gsize  leng,
-                   const gchar *charset,
-                   gboolean    *err)
+file_path_convert (const gchar  *file,
+                   const gssize  leng,
+                   const gchar  *charset,
+                   gboolean     *err)
 {
   gchar *result = NULL, *tmp;
-  gsize bytes_done = 0;
+  gsize bytes_done = 0, length;
 
   if (err)
     *err = TRUE;
-  if (!file || leng <= 0 || !charset)
+  if (!file || leng == 0 || !charset)
     return NULL;
-  while (bytes_done < leng)
+  length = leng < 0 ? g_strlen (file) : leng;
+  while (bytes_done < length)
     {
       if (file[bytes_done] == '\\')
         {
@@ -294,7 +101,7 @@ file_path_convert (const gchar *file,
           gchar *str = NULL;
           gsize len = 1;
 
-          for (len = 1;bytes_done + len <= leng; len++)
+          for (len = 1; bytes_done + len <= length; len++)
             {
               str = g_convert (file + bytes_done, len, "UTF-8", charset,
                                                             NULL, NULL, NULL);
@@ -324,12 +131,14 @@ file_path_convert (const gchar *file,
 
 /*  ja:ファイルの種類を取得する
     file,ファイル名
-     RET,FILE_KIND_DB:db,FILE_KIND_TAG:ID3TAG,FILE_KIND_UNKNOWN:不明        */
+     RET,FILE_KIND_TAG:ID3TAG,FILE_KIND_DB:db,FILE_KIND_LIST:プレイリスト   */
 guint
 file_is_type (const gchar *file)
 {
-  guint8 header[0xa0c0];
-  guint8 tag[128];
+  gchar *buf = NULL, *dir, **ary, **charset;
+  guint8 header[0xa0c0], tag[128];
+  gint i;
+  gssize fsize;
   FileIO *fio;
 
   fio = fileio_open (file, FILEIO_ACCESS_READ, FILEIO_SHARE_READ,
@@ -343,15 +152,68 @@ file_is_type (const gchar *file)
       fileio_close (fio);
       return FILE_KIND_DB;
     }
-  if (fileio_seek (fio, -128, FILEIO_SEEK_END) == -1
-                    || fileio_read (fio, tag, sizeof (tag)) != sizeof (tag))
+  if (fileio_seek (fio, -128, FILEIO_SEEK_END) != -1
+                    && fileio_read (fio, tag, sizeof (tag)) == sizeof (tag)
+                    && tag[0] == 'T' && tag[1] == 'A' && tag[2] == 'G'
+                    && (tag[126] == 0 || tag[126] == ' '))
     {
+      g_free (buf);
       fileio_close (fio);
-      return FILE_KIND_UNKNOWN;
+      return FILE_KIND_TAG;
+    }
+  fsize = fileio_seek (fio, 0, FILEIO_SEEK_END);
+  if (fsize < 0 || !(buf = g_malloc0 (fsize + sizeof (gchar)))
+                                || fileio_seek (fio, 0, FILEIO_SEEK_SET) == -1
+                                || fileio_read (fio, buf, fsize) != fsize)
+    {
+      g_free (buf);
+      fileio_close (fio);
+      return  FILE_KIND_UNKNOWN;
     }
   fileio_close (fio);
-  return tag[0] == 'T' && tag[1] == 'A' && tag[2] == 'G'
-    && (tag[126] == 0 || tag[126] == ' ') ? FILE_KIND_TAG : FILE_KIND_UNKNOWN;
+  /* ja:キャラクターセット変換 */
+  dir = g_path_get_dirname (file);
+  ary = g_strsplit (buf, "\r\n", G_MAXINT);
+  charset = g_strsplit (charset_list, ",", G_MAXINT);
+  g_free (buf);
+  for (i = 0; charset[i]; i++)
+    {
+      gint j;
+
+      for (j = 0; ary[j]; j++)
+        if (*(ary[j]) != '\0' && *(ary[j]) != '#'
+                                        && *(ary[j]) != G_SEARCHPATH_SEPARATOR)
+          {
+            gboolean err;
+            gchar *str;
+
+            str = file_path_convert (ary[j], -1, charset[i], &err);
+            if (str && err)
+              {
+                gchar *path;
+
+                path = g_path_is_absolute (str) ? g_strdup (str)
+                            : g_strconcat (dir, G_DIR_SEPARATOR_S, str, NULL);
+                if (!fileio_isfile (path))
+                  err = FALSE;
+                g_free (path);
+              }
+            g_free (str);
+            if (!err)
+              break;
+          }
+      if (!ary[j])
+        {
+          g_strfreev (charset);
+          g_strfreev (ary);
+          g_free (dir);
+          return  FILE_KIND_LIST;
+        }
+    }
+  g_strfreev (charset);
+  g_strfreev (ary);
+  g_free (dir);
+  return  FILE_KIND_UNKNOWN;
 }
 
 
@@ -499,9 +361,11 @@ file_open_id3tag (const gchar *file,
 
 
 /*  ja:dbファイルを開く
+     file,ファイル名
     mmaid,ウインドウ情報                                                    */
 void
-file_open_db (MmaidWindow *mmaid)
+file_open_db (const gchar *file,
+              MmaidWindow *mmaid)
 {
   gchar **charset, *dir, *tmp;
   guint8 header[0xa0c0];
@@ -517,7 +381,7 @@ file_open_db (MmaidWindow *mmaid)
   dialog = abort_dialog (_("Reading"));
   gtk_grab_add (dialog);
   /* ja:読み込み */
-  fio = fileio_open (mmaid->file, FILEIO_ACCESS_READ, FILEIO_SHARE_READ,
+  fio = fileio_open (file, FILEIO_ACCESS_READ, FILEIO_SHARE_READ,
                                                     FILEIO_MODE_OPEN_EXISTING);
   if (!fio || fileio_read (fio, header, sizeof (header)) != sizeof (header)
         || g_memcmp (irivdb_header0, header, sizeof (irivdb_header0))
@@ -659,7 +523,9 @@ file_open_db (MmaidWindow *mmaid)
     for (j = 0; j < 5; j++)
       g_free (mi_r[i].str[j]);
   /* ja:リストに追加 */
-  tmp = g_path_get_dirname (mmaid->file);
+  dir = fileio_get_full_path (file);
+  tmp = g_path_get_dirname (dir);
+  g_free (dir);
   dir = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
   g_free (tmp);
   for (i = 0; i < entries; i++)
@@ -683,32 +549,132 @@ file_open_db (MmaidWindow *mmaid)
 }
 
 
+/*  ja:プレイリストを開く
+     file,ファイル名
+    mmaid,ウインドウ情報                                                    */
+void
+file_open_list (const gchar *file,
+                MmaidWindow *mmaid)
+{
+  gchar *buf = NULL, *dir, **ary, **charset;
+  gint i, length, plist = 0;
+  gssize fsize;
+  FileIO *fio;
+  GList *glist = NULL;
+  GtkWidget *dialog;
+
+  /* ja:ダイアログボックス */
+  userbreak = TRUE;
+  dialog = abort_dialog (_("Reading"));
+  gtk_grab_add (dialog);
+  /* ja:読み込み */
+  fio = fileio_open (file, FILEIO_ACCESS_READ, FILEIO_SHARE_READ,
+                                                    FILEIO_MODE_OPEN_EXISTING);
+  fsize = fileio_seek (fio, 0, FILEIO_SEEK_END);
+  if (fsize < 0 || !(buf = g_malloc0 (fsize + sizeof (gchar)))
+                                || fileio_seek (fio, 0, FILEIO_SEEK_SET) == -1
+                                || fileio_read (fio, buf, fsize) != fsize)
+    {
+      g_free (buf);
+      fileio_close (fio);
+      /* ja:終了処理 */
+      gtk_grab_remove (dialog);
+      gtk_widget_destroy (dialog);
+      return;
+    }
+  fileio_close (fio);
+  /* ja:キャラクターセット変換 */
+  dir = g_path_get_dirname (file);
+  ary = g_strsplit (buf, "\r\n", G_MAXINT);
+  charset = g_strsplit (mmaid->charset, ",", G_MAXINT);
+  g_free (mmaid->charset);
+  g_free (buf);
+  for (i = 0; charset[i]; i++)
+    {
+      gint j;
+
+      plist = 0;
+      for (j = 0; ary[j]; j++)
+        if (*(ary[j]) != '\0' && *(ary[j]) != '#'
+                                        && *(ary[j]) != G_SEARCHPATH_SEPARATOR)
+          {
+            gboolean err;
+            gchar *str;
+
+            str = file_path_convert (ary[j], -1, charset[i], &err);
+            if (str && err)
+              {
+                gchar *path;
+
+                if (g_path_is_absolute (str))
+                  {
+                    path = g_strdup (str);
+                    plist = 1;
+                  }
+                else
+                  {
+                    path = g_strconcat (dir, G_DIR_SEPARATOR_S, str, NULL);
+                  }
+                if (fileio_isfile (path))
+                  {
+                    glist = g_list_append (glist, path);
+                  }
+                else
+                  {
+                    g_free (path);
+                    err = FALSE;
+                  }
+              }
+            g_free (str);
+            if (!err)
+              break;
+          }
+      if (!ary[j])
+        break;
+      for (j = g_list_length (glist) - 1; j >= 0; j--)
+        g_free (g_list_nth_data (glist, j));
+      g_list_free (glist);
+      glist = NULL;
+    }
+  mmaid->charset = g_strdup (charset[i] ? charset[i] : charset[0]);
+  g_strfreev (charset);
+  g_strfreev (ary);
+  g_free (dir);
+  length = g_list_length (glist);
+  if (length > 0)
+    mmaid->playlist = plist;
+  for (i = 0; i < length; i++)
+    {
+      gchar *path;
+      ID3Tag *id3tag;
+
+      path = g_list_nth_data (glist, i);
+      id3tag = g_malloc0 (sizeof (ID3Tag));
+      id3tag->file  = fileio_get_full_path (path);
+      id3tag->track = -1;
+      mm_list_append (MM_LIST (mmaid->list), id3tag);
+      g_free (path);
+    }
+  g_list_free (glist);
+  /* ja:終了処理 */
+  gtk_grab_remove (dialog);
+  gtk_widget_destroy (dialog);
+}
+
+
 /*  ja:ファイルを開く
     file,ファイル名
      RET,ウインドウ情報                                                     */
 MmaidWindow *
 file_open_edit (const gchar *file)
 {
-  gchar *text, *utf8str;
+  gchar *filename;
+  gint page_num;
   MmaidWindow *mmaid;
-  GdkBitmap *mask;
-  GdkPixmap *pixmap;
-  GtkWidget *menu_shell, *scroll, *hbox, *button, *frame;
-  static gint fcount = 0; /* ja:新規ファイルの数 */
-  static gchar *xpm[]={
-"8 7 2 1",
-"   c None",
-".  c #000000",
-"..    ..",
-" ..  .. ",
-"  ....  ",
-"   ..   ",
-"  ....  ",
-" ..  .. ",
-"..    .."};
+  GtkWidget *scroll;
 
   mmaid = g_malloc0 (sizeof (MmaidWindow));
-  mmaid->same = -1;
+  mmaid->playlist = playlist;
   /* ja:リスト */
   mmaid->list = mm_list_new ();
   mm_list_set_editable (MM_LIST (mmaid->list), TRUE);
@@ -716,21 +682,27 @@ file_open_edit (const gchar *file)
   if (!file)
     {
       gchar **charset;
+      static gint fcount = 0; /* ja:新規ファイルの数 */
 
       /* ja:新規 */
-      mmaid->file = g_strdup_printf ("new%04d", fcount++ % 1000);
-      mmaid->create = TRUE;
+      filename = g_strdup_printf ("new%04d.idb", fcount++ % 1000);
       charset = g_strsplit (charset_list, ",", G_MAXINT);
       mmaid->charset = g_strdup (charset[0]);
       g_strfreev (charset);
+      mmaid->filekind = FILE_KIND_DB;
     }
   else
     {
-      mmaid->file = fileio_get_full_path (file);
-      mmaid->create = FALSE;
+      filename = fileio_get_full_path (file);
       mmaid->charset = g_strdup (charset_list);
-      file_open_db (mmaid);
+      mmaid->filekind = file_is_type (file);
+      if (mmaid->filekind == FILE_KIND_LIST)
+        file_open_list (file, mmaid);
+      else
+        file_open_db (file, mmaid);
     }
+  g_signal_connect (G_OBJECT (mmaid->list), "button-press-event",
+                                    G_CALLBACK (signal_button_press), mmaid);
   g_signal_connect (G_OBJECT (mmaid->list), "changed",
                                     G_CALLBACK (signal_changed), mmaid);
   g_signal_connect (G_OBJECT (mmaid->list), "clicked",
@@ -745,56 +717,24 @@ file_open_edit (const gchar *file)
                                     G_CALLBACK (signal_moved), mmaid);
   g_signal_connect (G_OBJECT (mmaid->list), "destroy",
                                     G_CALLBACK (signal_destroy_list), mmaid);
-
-  /* ja:ファイル名を調べる */
-  text = file_add_edit (mmaid->file, &mmaid->same);
+  gtk_widget_add_events (mmaid->list, GDK_BUTTON_PRESS_MASK);
 
   /* ja:スクロールウインドウ */
   scroll = gtk_scrolled_window_new (NULL, NULL);
   gtk_container_add (GTK_CONTAINER (scroll), mmaid->list);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  g_object_set_data (G_OBJECT (scroll), "user_data", mmaid);
-  /* ja:メニュー */
-  menu_shell = gtk_item_factory_get_widget (ifactory_menu, "<main>/Window");
-  utf8str = g_filename_to_utf8 (mmaid->file, -1, NULL, NULL, NULL);
-  mmaid->menu_item = gtk_menu_item_new_with_label (utf8str);
-  g_free (utf8str);
-  g_signal_connect (G_OBJECT (mmaid->menu_item), "activate",
-                            G_CALLBACK (signal_activate_menu_window), scroll);
-  gtk_menu_append (GTK_MENU (menu_shell), mmaid->menu_item);
-  gtk_widget_show (mmaid->menu_item);
-  /* ja:ボタン */
-  button = gtk_button_new ();
-  frame = gtk_frame_new (NULL);
-  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask, NULL, xpm);
-  gtk_container_add (GTK_CONTAINER (frame), gtk_pixmap_new (pixmap, mask));
-  gtk_container_add (GTK_CONTAINER (button), frame);
-  g_signal_connect (G_OBJECT (button), "clicked",
-                                G_CALLBACK (signal_clicked_button), scroll);
-  /* ja:ラベル */
-  mmaid->label = gtk_label_new (text);
-  g_free (text);
-  /* ja:マーク */
-  mmaid->mark = gtk_label_new ("*");
   /* ja:表示 */
-  hbox = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), mmaid->label, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), mmaid->mark, FALSE, FALSE, 0);
-  gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), scroll, hbox);
-  gtk_widget_show (hbox);
-  gtk_widget_show (mmaid->label);
-  gtk_widget_show_all (button);
+  page_num = misc_mdi_append_page (MISC_MDI (mdi),
+                                            scroll, filename, !file, mmaid);
   gtk_widget_show_all (scroll);
-  gtk_notebook_set_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_page_num (GTK_NOTEBOOK (notebook),scroll));
+  gtk_notebook_set_page (GTK_NOTEBOOK (mdi), page_num);
   gtk_widget_grab_focus (mmaid->list);
 
   /* ja:新規作成ではなく、ファイルが存在する時には履歴に加える */
-  if (file && history > 0 && fileio_isfile (mmaid->file))
-    file_set_history (mmaid->file);
+  if (file && fileio_isfile (file))
+    misc_history_add_file (MISC_HISTORY (history), file);
+  g_free (filename);
 
   return mmaid;
 }
@@ -1095,3 +1035,137 @@ file_save_db (const gchar *file,
   gtk_widget_destroy (dialog);
   return userbreak;
 }
+
+
+/*  ja:プレイリストを保存する
+        file,ファイル名
+       mmaid,ウインドウ情報
+        root,ルート
+     charset,キャラクターセット
+    playlist.プレイリスト(0:相対,1:絶対,2:名前)
+         RET,TRUE:正常終了,FALSE:エラー                                     */
+gboolean
+file_save_list (const gchar *file,
+                MmaidWindow *mmaid,
+                const gchar *root,
+                const gchar *charset,
+                const gint   playlist)
+{
+  FileIO *fio;
+
+  fio = fileio_open (file, FILEIO_ACCESS_READ | FILEIO_ACCESS_WRITE,
+                                FILEIO_SHARE_READ, FILEIO_MODE_CREATE_ALWAYS);
+  if (fio)
+    {
+      gint i, count;
+      GtkWidget *dialog;
+
+      /* ja:ダイアログボックス */
+      userbreak = TRUE;
+      dialog = abort_dialog (_("Writing"));
+      gtk_grab_add (dialog);
+      count = mm_list_length (MM_LIST (mmaid->list));
+      for (i = 0; i < count && userbreak; i++)
+        {
+          gchar *element, *tmp, **ary;
+          gint j;
+          const ID3Tag *id3tag;
+
+          while (gtk_events_pending ())
+            gtk_main_iteration ();
+          id3tag = mm_list_get_nth (MM_LIST (mmaid->list), i);
+          switch (playlist)
+            {
+              case 0:/* ja:相対パス */
+                {
+                  gchar *file, *relative;
+
+                  file = g_filename_from_utf8 (id3tag->file, -1,
+                                                            NULL, NULL, NULL);
+                  relative = fileio_get_relative_path (file, root);
+                  g_free (file);
+                  element = g_filename_to_utf8 (relative, -1,
+                                                            NULL, NULL, NULL);
+                  g_free (relative);
+                }
+                break;
+              case 1:/* ja:絶対パス */
+                element = g_strdup (id3tag->file);
+                break;
+              default:/* ja:ファイル名のみ */
+                {
+                  gchar *file, *name;
+
+                  file = g_filename_from_utf8 (id3tag->file, -1,
+                                                            NULL, NULL, NULL);
+                  name = g_path_get_basename (file);
+                  g_free (file);
+                  element = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
+                  g_free (name);
+                }
+            }
+          tmp = g_strconcat (element, "\r\n", NULL);
+          g_free (element);
+          element = NULL;
+          ary = g_strsplit (tmp, G_DIR_SEPARATOR_S, G_MAXINT);
+          g_free (tmp);
+          for (j = 0; ary[j]; j++)
+            {
+              gchar *name;
+
+              name = g_convert (ary[j], -1,
+                                        charset, "UTF-8", NULL, NULL, NULL);
+              if (!name)
+                {
+                  gchar *text;
+
+                  g_free (element);
+                  text = g_strdup_printf
+                                        (_("Conversion from character encoding"
+                                        " 'UTF-8' to '%s' is not supported"),
+                                                            charset);
+                  misc_message_box ("Melody maid", text, 0, _("OK"), NULL);
+                  g_free (text);
+                  userbreak = FALSE;
+                  break;
+                }
+              if (element)
+                {
+                  tmp = g_strconcat (element, "\\", name, NULL);
+                  g_free (name);
+                  g_free (element);
+                  element = tmp;
+                }
+              else
+                {
+                  element = name;
+                }
+            }
+          g_strfreev (ary);
+          if (element)
+            {
+              gsize leng;
+
+              leng = g_strlen (element) * sizeof (gchar);
+              if (fileio_write (fio, element, leng) != leng)
+                {
+                  misc_message_box ("Melody maid",
+                                        _("Can not write"), 0, _("OK"), NULL);
+                  userbreak = FALSE;
+                }
+              g_free (element);
+            }
+        }
+      /* ja:終了処理 */
+      gtk_grab_remove (dialog);
+      gtk_widget_destroy (dialog);
+      fileio_close (fio);
+    }
+  else
+    {
+      misc_message_box ("Melody maid", _("Can not open file"),
+                                                            0, _("OK"), NULL);
+      userbreak = FALSE;
+    }
+  return userbreak;
+}
