@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,8 +30,9 @@ extern "C" {
 
 #define FILE_KIND_ERROR    0
 #define FILE_KIND_UNKNOWN  1
-#define FILE_KIND_DB       2
-#define FILE_KIND_TAG      3
+#define FILE_KIND_TAG      2
+#define FILE_KIND_DB       3
+#define FILE_KIND_LIST     4
 
 
 #define FILE_SAVE_SUCCESS          0
@@ -48,31 +49,6 @@ extern "C" {
 #define FILE_SAVE_LENGTH_COMMENT  11
 
 
-/******************************************************************************
-*                                                                             *
-* ja:ファイル関数群                                                           *
-*                                                                             *
-******************************************************************************/
-/*  ja:ファイルの履歴に加える
-    file,ファイル名                                                         */
-void
-file_set_history (const gchar *file);
-
-
-/*  ja:ファイルを加える
-    file,加えようとするファイルのフルパス
-     RET,加えようとするファイルのラベル                                     */
-gchar *
-file_add_edit (const gchar *file,
-               gint        *same);
-
-
-/*  ja:ファイルを削除する
-    file,削除しようとするファイルのフルパス                                 */
-void
-file_delete_edit (const gchar *file);
-
-
 /******************************************************************************
 *                                                                             *
 * ファイル入力関数群                                                          *
@@ -80,7 +56,7 @@ file_delete_edit (const gchar *file);
 ******************************************************************************/
 /*  ja:ファイルの種類を取得する
     file,ファイル名
-     RET,FILE_KIND_DB:db,FILE_KIND_TAG:ID3TAG,FILE_KIND_UNKNOWN:不明        */
+     RET,FILE_KIND_TAG:ID3TAG,FILE_KIND_DB:db,FILE_KIND_LIST:プレイリスト   */
 guint
 file_is_type (const gchar *file);
 
@@ -95,9 +71,19 @@ file_open_id3tag (const gchar *file,
 
 
 /*  ja:dbファイルを開く
+     file,ファイル名
     mmaid,ウインドウ情報                                                    */
 void
-file_open_db (MmaidWindow *mmaid);
+file_open_db (const gchar *file,
+              MmaidWindow *mmaid);
+
+
+/*  ja:プレイリストを開く
+     file,ファイル名
+    mmaid,ウインドウ情報                                                    */
+void
+file_open_list (const gchar *file,
+                MmaidWindow *mmaid);
 
 
 /*  ja:ファイルを開く
@@ -136,6 +122,21 @@ file_save_db (const gchar *file,
               const gchar *charset);
 
 
+/*  ja:プレイリストを保存する
+        file,ファイル名
+       mmaid,ウインドウ情報
+        root,ルート
+     charset,キャラクターセット
+    playlist.プレイリスト(0:相対,1:絶対,2:名前)
+         RET,TRUE:正常終了,FALSE:エラー                                     */
+gboolean
+file_save_list (const gchar *file,
+                MmaidWindow *mmaid,
+                const gchar *root,
+                const gchar *charset,
+                const gint   playlist);
+
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
