@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,9 @@
 #include "general.h"
 #include "mm_list.h"
 #include "toolbar.h"
+#include "misc/charuty.h"
 #include "misc/misc.h"
+#include "misc/misc_mdi.h"
 
 
 /******************************************************************************
@@ -33,12 +35,10 @@
 void
 set_menu_bar (MmaidWindow *mmaid)
 {
-  GList *glist;
-  GtkWidget *menu;
-
   if (mmaid)
     {
       gboolean arrow_up = FALSE, arrow_down = FALSE, arrow_select = FALSE;
+      gboolean created;
       gint i, count;
 
       count = mm_list_length (MM_LIST (mmaid->list));
@@ -83,6 +83,8 @@ set_menu_bar (MmaidWindow *mmaid)
                   }
             }
         }
+      created = misc_mdi_get_created (MISC_MDI (mdi),
+                        misc_mdi_get_page_from_data (MISC_MDI (mdi), mmaid));
       gtk_widget_set_sensitive (gtk_menu_get_attach_widget (GTK_MENU
             (gtk_item_factory_get_widget (ifactory_menu, "<main>/Tag"))),
                                                                         TRUE);
@@ -93,7 +95,7 @@ set_menu_bar (MmaidWindow *mmaid)
       gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
                                 "<main>/File/Save As..."), count);
       gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
-                                "<main>/File/Reload..."), !mmaid->create);
+                                "<main>/File/Reload..."), !created);
       gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
                                 "<main>/File/Export..."), arrow_select);
       gtk_widget_set_sensitive (gtk_item_factory_get_widget (ifactory_menu,
@@ -157,11 +159,6 @@ set_menu_bar (MmaidWindow *mmaid)
       gtk_widget_set_sensitive (toolbar_items[15].item, FALSE);
       gtk_widget_set_sensitive (toolbar_items[16].item, FALSE);
     }
-  menu = gtk_item_factory_get_widget (ifactory_menu, "<main>/Window");
-  glist = gtk_container_children (GTK_CONTAINER (menu));
-  gtk_widget_set_sensitive (gtk_menu_get_attach_widget (GTK_MENU (menu)),
-                                                    g_list_length (glist) > 0);
-  g_list_free (glist);
 }
 
 
@@ -171,11 +168,15 @@ set_menu_bar (MmaidWindow *mmaid)
 gboolean
 prompt_close (MmaidWindow *mmaid)
 {
-  if (mmaid->edit)
+  gint page;
+
+  page = misc_mdi_get_page_from_data (MISC_MDI (mdi), mmaid);
+  if (misc_mdi_get_edited (MISC_MDI (mdi), page))
     {
       gchar *text, *utf8str;
 
-      utf8str = g_filename_to_utf8 (mmaid->file, -1, NULL, NULL, NULL);
+      utf8str = g_filename_to_utf8
+            (misc_mdi_get_file (MISC_MDI (mdi), page), -1, NULL, NULL, NULL);
       text = g_strdup_printf (_("File %s was edited.\nSave?"), utf8str);
       g_free (utf8str);
       switch (misc_message_box ("Melody maid", text, 0,
@@ -183,35 +184,17 @@ prompt_close (MmaidWindow *mmaid)
         {
           case 0:
             {
-              GtkWidget *child;
+              gint page_num;
 
-              child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)));
-              if (g_object_get_data (G_OBJECT (child), "user_data") != mmaid)
-                {
-                  gint i;
-                  GList *glist;
-
-                  glist = gtk_container_children (GTK_CONTAINER (notebook));
-                  for (i = g_list_length (glist) - 1; i >= 0; i--)
-                    if (g_object_get_data (G_OBJECT (g_list_nth_data
-                                            (glist, i)), "user_data") == mmaid)
-                      {
-                        gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook),
-                                                                            i);
-                        break;
-                      }
-                  g_list_free (glist);
-                }
+              page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+              if (page != page_num)
+                gtk_notebook_set_current_page (GTK_NOTEBOOK (mdi), page);
               else
-                {
-                  child = NULL;
-                }
+                page_num = -1;
               command_save (NULL, 0, NULL);
-              if (child)
-                gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_page_num (GTK_NOTEBOOK (notebook), child));
-              if (!mmaid->edit)
+              if (page_num >= 0)
+                gtk_notebook_set_current_page (GTK_NOTEBOOK (mdi), page_num);
+              if (!misc_mdi_get_edited (MISC_MDI (mdi), page))
                 break;
             }
           case 2:
@@ -225,20 +208,34 @@ prompt_close (MmaidWindow *mmaid)
 }
 
 
-/*  ja:タブの編集済みマークの変更
-      mmaid,ウインドウ情報
-    is_edit,TRUE:編集済み,FALSE:未編集                                      */
+/*  ja:キャラクターセットを置換する
+    charlist,置換するキャラクターセット                                     */
 void
-mmaid_change_edit (MmaidWindow *mmaid,
-                   gboolean     is_edit)
+charlist_renewal_all (const gchar *charlist)
 {
-  if (mmaid->edit != is_edit)
+  gchar *tmp;
+  gint i;
+
+  if (!charlist)
+    return;
+  if (charset_list != charlist)
     {
-      mmaid->edit = is_edit;
-      if (is_edit)
-        gtk_widget_show (mmaid->mark);
-      else
-        gtk_widget_hide (mmaid->mark);
+      tmp = charuty_append (charset_list, charlist);
+      g_free (charset_list);
+      charset_list = tmp;
+    }
+  for (i = misc_mdi_get_n_pages (MISC_MDI (mdi)) - 1; i >= 0; i--)
+    {
+      MmaidWindow *mmaid;
+
+      mmaid = misc_mdi_get_data (MISC_MDI (mdi), i);
+      if (mmaid->charset != charlist)
+        {
+          tmp = charuty_rename (mmaid->charset, charlist);
+          g_free (mmaid->charset);
+          mmaid->charset = tmp;
+        }
+      mm_list_charset_renewal (MM_LIST (mmaid->list), charlist);
     }
 }
 
@@ -256,7 +253,7 @@ gboolean
 cmp_id3tag (const ID3Tag *id3tag0,
             const ID3Tag *id3tag1)
 {
-  return !g_strfilecmp (id3tag0->file, id3tag1->file)
+  return g_strfilecmp (id3tag0->file, id3tag1->file) == 0
       && ((!id3tag0->title   && !id3tag1->title)
                         || (id3tag0->title   && id3tag1->title
                         && !g_strcmp (id3tag0->title,   id3tag1->title)))
