@@ -1,5 +1,5 @@
 /*
-    Meload maid
+    Melody maid
     copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
