@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,8 +16,9 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-#include "charlist.h"
+#include "general.h"
 #include "mm_list.h"
+#include "misc/charuty.h"
 #include "misc/misc.h"
 #include <gdk/gdkkeysyms.h>
 
@@ -282,7 +283,7 @@ mm_list_edited (GtkCellRendererText *cell,
             return;
         break;
       case MM_LIST_CHARSET:
-        if (text && text[0] != '\0' && !charlist_is_valid (text))
+        if (text && text[0] != '\0' && !charuty_is_valid (text))
           return;
     }
   path = gtk_tree_path_new_from_string (path_string);
@@ -537,7 +538,7 @@ mm_list_insert (MmList     *list,
 
 /*  ja:項目を探す
     list,ウィジェット
-     ...,コラム(gint)と比較内容,-1で終了
+     ...,コラム(gint)と比較内容,MM_LIST_INVALIDで終了
      RET,項目,-1:項目なし                                                   */
 gint
 mm_list_search (MmList *list,
@@ -623,7 +624,7 @@ mm_list_search (MmList *list,
 /*  ja:項目を変更する
     list,ウィジェット
        n,項目(この項目の直後に挿入,-1:先頭)
-     ...,コラム(gint)と変更内容,-1で終了                                    */
+     ...,コラム(gint)と変更内容,MM_LIST_INVALIDで終了                       */
 void
 mm_list_change (MmList     *list,
                 const gint  n,
@@ -947,7 +948,7 @@ mm_list_get_nth (MmList     *list,
 /*  ja:カーソル位置を取得する
     list,ウィジェット
        n,カーソル位置,-1:カーソル位置なし
-       c,コラム位置,-1:コラム位置なし                                       */
+       c,コラム位置,MM_LIST_INVALID:コラム位置なし                          */
 void
 mm_list_get_cursor (MmList *list,
                     gint   *n,
@@ -989,14 +990,14 @@ mm_list_get_cursor (MmList *list,
     gtk_tree_path_free (path);
   if (c)
     *c = column ? (gint)g_object_get_data (G_OBJECT (column), "user_data")
-                : -1;
+                : MM_LIST_INVALID;
 }
 
 
 /*  ja:カーソル位置を設定する
     list,ウィジェット
        n,カーソル位置,-1:カーソル位置なし
-       c,コラム位置,-1:コラム位置なし                                       */
+       c,コラム位置,MM_LIST_INVALID:コラム位置なし                          */
 void
 mm_list_set_cursor (MmList     *list,
                     const gint  n,
@@ -1036,6 +1037,70 @@ mm_list_set_cursor (MmList     *list,
 }
 
 
+/*  ja:座標から位置を取得する
+    list,ウィジェット
+       x,X座標
+       y,Y座標
+       n,カーソル位置,-1:カーソル位置なし
+       c,コラム位置,-1:コラム位置なし                                       */
+void
+mm_list_get_cursor_from_pos (MmList     *list,
+                             const gint  x,
+                             const gint  y,
+                             gint       *n,
+                             gint       *c)
+{
+  GtkTreePath *path;
+  GtkTreeViewColumn *column;
+
+  if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (list), x, y,
+                                                &path, &column, NULL, NULL))
+    {
+      if (n)
+        {
+          if (path)
+            {
+              GtkTreeIter iter;
+              ID3Tag *id3tag;
+
+              gtk_tree_model_get_iter (GTK_TREE_MODEL (list->store), &iter,
+                                                                        path);
+              gtk_tree_model_get (GTK_TREE_MODEL (list->store), &iter,
+                                                MM_LIST_ID3TAG, &id3tag, -1);
+              for (*n = gtk_tree_model_iter_n_children
+                    (GTK_TREE_MODEL (list->store), NULL) - 1; *n >= 0; (*n)--)
+                {
+                  ID3Tag *id3tag0;
+
+                  gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (list->store),
+                                                            &iter, NULL, *n);
+                  gtk_tree_model_get (GTK_TREE_MODEL (list->store), &iter,
+                                                MM_LIST_ID3TAG, &id3tag0, -1);
+                  if (id3tag == id3tag0)
+                    break;
+                }
+            }
+          else
+            {
+              *n = -1;
+            }
+        }
+      if (path)
+        gtk_tree_path_free (path);
+      if (c)
+        *c = column ? (gint)g_object_get_data (G_OBJECT (column), "user_data")
+                    : MM_LIST_INVALID;
+    }
+  else
+    {
+      if (n)
+        *n = -1;
+      if (c)
+        *c = MM_LIST_INVALID;
+    }
+}
+
+
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
@@ -1589,7 +1654,7 @@ mm_list_charset_renewal (MmList      *list,
           gchar *tmp;
 
           tmp = id3tag->charset;
-          id3tag->charset = charlist_rename (id3tag->charset, charlist);
+          id3tag->charset = charuty_rename (id3tag->charset, charlist);
           gtk_list_store_set (list->store, &iter,
                                         MM_LIST_CHARSET, id3tag->charset, -1);
           g_free (tmp);
