@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,45 +16,51 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
+#include <locale.h>
 #include "mmaid.h"
-#include "charlist.h"
 #include "command.h"
 #include "file.h"
 #include "general.h"
 #include "genre.h"
 #include "mm_list.h"
+#include "process.h"
 #include "sigmain.h"
 #include "toolbar.h"
 #include "version.h"
 #include "misc/argument.h"
+#include "misc/charuty.h"
 #include "misc/fileio.h"
 #include "misc/license.h"
 #include "misc/misc_accel.h"
 #include "misc/misc_comm.h"
+#include "misc/misc_history.h"
+#include "misc/misc_mdi.h"
+#include "misc/misc_winmenu.h"
 #include "misc/profile.h"
 
 
 gboolean newfile = FALSE;           /* ja:TRUE:新規ファイル,FALSE:なし */
 gboolean second = FALSE;            /* ja:TRUE:新規,FALSE:既存 */
 gboolean def_hold = TRUE;           /* ja:TRUE:保持,FALSE:上書き */
-gchar *history_files[10];           /* ja:ファイルの履歴 */
 gchar *open_path = NULL;            /* ja:開くのパス */
 gchar *save_path = NULL;            /* ja:名前を付けて保存のパス */
 gchar *clipboard_id3tag = NULL;     /* ja:クリップボードのタグ */
 gchar *charset_list = NULL;         /* ja:キャラクターセットリスト */
+gchar *cmd_play = NULL;             /* ja:演奏コマンド */
+gchar *cmd_stop = NULL;             /* ja:停止コマンド */
 gchar *genre[256][256];             /* ja:ジャンル */
 gint genres = 0;                    /* ja:ジャンルの数 */
 gint def_gnum = 192;                /* ja:デフォルトのジャンル */
 gint def_width, def_height;         /* ja:デフォルトのウインドウのサイズ */
-gint history = 4;                   /* ja:ファイルの履歴 */
+gint playlist = 0;                  /* ja:プレイリスト */
+GObject *history;                   /* ja:ファイルの履歴 */
 GdkAtom atom_targets;               /* ja:ターゲット */
 GdkAtom atom_id3tag;                /* ja:ID3タグ */
 GtkWidget *window;                  /* ja:ウインドウ */
-GtkWidget *notebook;                /* ja:ノートブック */
+GtkWidget *mdi;                     /* en:MDI */
 GtkPositionType n_pos = GTK_POS_TOP;/* ja:ノートブックの位置 */
 GtkItemFactory *ifactory_menu;      /* ja:メニュー */
 GtkItemFactory *ifactory_popup;     /* ja:ポップアップ */
-GtkListStore *list_store;           /* ja:リスト */
 GtkTargetEntry targets_mmaid[] = {  /* ja:提供可能なセレクション */
     {"Melody maid ID3 Tag", 0, TARGET_MMAID}};
 #ifdef G_OS_WIN32
@@ -84,10 +90,9 @@ main (int   argc,
   gint i, j, count;
   MmaidWindow *mmaid = NULL;
   Profile *profile;
-  GObject *accel;
+  GObject *accel, *winmenu;
   GtkAccelGroup *accel_group;
-  GtkWidget *menu_bar, *menu_shell, *menu_item;
-  GtkWidget *handle, *toolbar, *vbox;
+  GtkWidget *menu_bar, *handle, *toolbar, *vbox;
   GtkItemFactoryEntry menu_items[] = {
 {N_("/_File"),                         NULL,              NULL,            0, "<Branch>"},
 {N_("/File/_New"),                     "<control>N",      command_new,     0, "<Item>"},
@@ -120,15 +125,13 @@ main (int   argc,
 {N_("/Option/_Configuration..."),      NULL,              command_conf,    0, "<Item>"},
 {N_("/Option/_Genre..."),              NULL,              genre_dialog,    0, "<Item>"},
 {N_("/Option/Character _Encoding..."), NULL,              command_charset, 0, "<Item>"},
+{N_("/Option/_Process..."),            NULL,              process_dialog,  0, "<Item>"},
 {N_("/_Window"),                       NULL,              NULL,            0, "<Branch>"},
 {N_("/_Help"),                         NULL,              NULL,            0, "<Branch>"},
 {N_("/Help/_About..."),                NULL,              version_dialog,  0, "<Item>"}};
     GtkItemFactoryEntry popup_items[] = {
-{N_("/R_oot..."),                      NULL,              command_root,    0, "<Item>"},
-{N_("/_Refresh..."),                   "F5",              command_refresh, 0, "<Item>"},
-{N_("/A_dd..."),                       "Insert",          command_add,     0, "<Item>"},
-{N_("/_Edit..."),                      "<control>Return", command_edit,    0, "<Item>"},
-{N_("/_Search..."),                    "<control>F",      command_search,  0, "<Item>"},
+{N_("/Play"),                          NULL,              command_play,    0, "<Item>"},
+{N_("/Stop"),                          NULL,              command_stop,    0, "<Item>"},
 {"/Sep1",                              NULL,              NULL,            0, "<Separator>"},
 {N_("/Cu_t"),                          "<control>X",      command_cut,     0, "<Item>"},
 {N_("/_Copy"),                         "<control>C",      command_copy,    0, "<Item>"},
@@ -163,19 +166,24 @@ main (int   argc,
 
   def_width  = gdk_screen_width ()  * 2 / 3;
   def_height = gdk_screen_height () * 2 / 3;
-  g_memset (history_files, 0, sizeof (gchar *) * 10);
   g_memset (genre, 0, sizeof (gchar *) * 256 * 256);
+
+  /* ja:ファイルの履歴 */
+  history = misc_history_new ();
+
   /* ja:レジストリから読み出す */
   profile = profile_open (NULL);
   if (profile)
     {
+      gint num;
+
       if (!profile_get_value (profile, "mmaid", "license",
                 (gpointer)&license, sizeof (gboolean),
                                                     PROFILE_VALUE_TYPE_BOOL))
         license = FALSE;
-      if (!profile_get_value (profile, "mmaid", "history",
-                (gpointer)&history, sizeof (gint), PROFILE_VALUE_TYPE_INT))
-        history = 4;
+      if (profile_get_value (profile, "mmaid", "history",
+                (gpointer)&num, sizeof (gint), PROFILE_VALUE_TYPE_INT))
+        misc_history_set_num (MISC_HISTORY (history), num);
       if (!profile_get_value (profile, "mmaid", "newfile",
                 (gpointer)&newfile, sizeof (gboolean),
                                                     PROFILE_VALUE_TYPE_BOOL))
@@ -199,12 +207,17 @@ main (int   argc,
       if (!profile_get_value (profile, "mmaid", "n_pos",
                 (gpointer)&n_pos, sizeof (gint), PROFILE_VALUE_TYPE_INT))
         n_pos = GTK_POS_TOP;
+      if (!profile_get_value (profile, "mmaid", "playlist",
+                (gpointer)&playlist, sizeof (gint), PROFILE_VALUE_TYPE_INT))
+        playlist = 0;
+      cmd_play = profile_get_string (profile, "mmaid", "cmd_play");
+      cmd_stop = profile_get_string (profile, "mmaid", "cmd_stop");
       charset_list = profile_get_string (profile, "mmaid", "charlist");
       if (charset_list)
         {
           gchar *tmp;
 
-          tmp = charlist_strip (charset_list);
+          tmp = charuty_strip (charset_list);
           g_free (charset_list);
           charset_list = tmp;
         }
@@ -217,21 +230,19 @@ main (int   argc,
                 ? g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL) : NULL;
       g_free (utf8str);
       /* ja:ファイルの履歴 */
-      for (i = 0; i < history; i++)
+      for (i = misc_history_get_num (MISC_HISTORY (history)) - 1; i >= 0; i--)
         {
           key = g_strdup_printf ("history%d", i);
           utf8str = profile_get_string (profile, "mmaid", key);
-          history_files[i] = utf8str
-                ? g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL) : NULL;
-          g_free (utf8str);
           g_free (key);
-          if (!history_files[i])
-            break;
-          if (*history_files[i] == '\0')
+          if (utf8str)
             {
-              g_free (history_files[i]);
-              history_files[i] = NULL;
-              break;
+              gchar *file;
+
+              file = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
+              g_free (utf8str);
+              misc_history_add_file (MISC_HISTORY (history), file);
+              g_free (file);
             }
         }
       /* ja:ジャンル */
@@ -319,7 +330,7 @@ main (int   argc,
 {NULL,      '\0', NULL,           NULL,              0,                   NULL}};
 
     /* ja:ファイルの履歴 */
-    arg_h.val = history;
+    arg_h.val = misc_history_get_num (MISC_HISTORY (history));
     arg_h.def = 4;
     /* ja:新規ファイル */
     arg_n.flag = newfile;
@@ -344,8 +355,7 @@ MMAID_COPYRIGHT_NOTICE"\n"
     /* ja:ヘルプ */
     arg_q.str = MMAID_COPYRIGHT_NOTICE"\n\n";
     arg_files = arg_analyse (argv, arg_opts);
-    if (0 <= arg_h.val && arg_h.val <= 10)
-      history = arg_h.val;
+    misc_history_set_num (MISC_HISTORY (history), arg_h.val);
     def_width = array[0];
     def_height = array[1];
     second = arg_x.flag;
@@ -358,7 +368,7 @@ MMAID_COPYRIGHT_NOTICE"\n"
   g_signal_connect (G_OBJECT (window), "delete-event",
                                             G_CALLBACK (signal_delete), NULL);
   g_signal_connect (G_OBJECT (window), "destroy",
-                                    G_CALLBACK (signal_destroy_window), NULL);
+                                            G_CALLBACK (gtk_main_quit), NULL);
   gtk_widget_realize (window);
   /* en:Drag and Drop */
   gtk_drag_dest_set (window, GTK_DEST_DEFAULT_MOTION
@@ -381,12 +391,14 @@ MMAID_COPYRIGHT_NOTICE"\n"
                                 G_CALLBACK (signal_selection_get), NULL);
   g_signal_connect (G_OBJECT (window), "selection-received",
                                 G_CALLBACK (signal_selection_received), NULL);
-  /* ja:ノートブック */
-  notebook = gtk_notebook_new ();
-  gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
-  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), n_pos);
-  g_signal_connect (G_OBJECT (notebook), "switch-page",
-                                G_CALLBACK (signal_switch_page), NULL);
+  /* en:MDI */
+  mdi = misc_mdi_new ();
+  gtk_notebook_set_scrollable (GTK_NOTEBOOK (mdi), TRUE);
+  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (mdi), n_pos);
+  g_signal_connect (G_OBJECT (mdi), "close-window",
+                                G_CALLBACK (signal_close_window), NULL);
+  g_signal_connect (G_OBJECT (mdi), "switch-window",
+                                G_CALLBACK (signal_switch_window), NULL);
   /* ja:ポップアップ */
   accel_group = gtk_accel_group_new ();
   ifactory_popup = gtk_item_factory_new (GTK_TYPE_MENU, "<main>", accel_group);
@@ -404,6 +416,16 @@ MMAID_COPYRIGHT_NOTICE"\n"
         sizeof(menu_items) / sizeof (GtkItemFactoryEntry), menu_items, NULL);
   gtk_widget_add_events (window, GDK_KEY_PRESS_MASK);
   menu_bar = gtk_item_factory_get_widget (ifactory_menu, "<main>");
+  /* ja:ファイルの履歴 */
+  misc_history_set_menu (MISC_HISTORY (history),
+    gtk_item_factory_get_widget (ifactory_menu, "<main>/File"), MENUFILE - 2);
+  g_signal_connect (MISC_HISTORY (history), "activate",
+                            G_CALLBACK (signal_activate_menu_history), NULL);
+  /* ja:ウインドウメニュー */
+  winmenu = misc_winmenu_new ();
+  misc_winmenu_set_mdi (MISC_WINMENU (winmenu), MISC_MDI (mdi));
+  misc_winmenu_set_menu (MISC_WINMENU (winmenu),
+            gtk_item_factory_get_widget (ifactory_menu, "<main>/Window"), 0);
   /* ja:アクセルグループ */
   accel = misc_accel_new ();
   misc_accel_set_window (MISC_ACCEL (accel), GTK_WINDOW (window));
@@ -444,25 +466,8 @@ MMAID_COPYRIGHT_NOTICE"\n"
   gtk_container_set_border_width (GTK_CONTAINER (handle), 2);
   gtk_container_add (GTK_CONTAINER (handle), toolbar);
   gtk_box_pack_start (GTK_BOX (vbox), handle, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), mdi, TRUE, TRUE, 0);
   gtk_container_add (GTK_CONTAINER (window), vbox);
-  /* ja:ファイルの履歴 */
-  menu_shell = gtk_item_factory_get_widget (ifactory_menu, "<main>/File");
-  for (i = 0; i < history && history_files[i]; i++)
-    {
-      utf8str = g_filename_to_utf8 (history_files[i], -1, NULL, NULL, NULL);
-      menu_item = gtk_menu_item_new_with_label (utf8str);
-      g_free (utf8str);
-      g_signal_connect (G_OBJECT (menu_item), "activate",
-                G_CALLBACK (signal_activate_menu_history), history_files[i]);
-      g_signal_connect (G_OBJECT (menu_item), "destroy",
-                G_CALLBACK (signal_destroy_menu_history), history_files[i]);
-      g_object_set_data (G_OBJECT (menu_item), "user_data", history_files[i]);
-      gtk_menu_insert (GTK_MENU (menu_shell), menu_item, i + MENUFILE - 1);
-    }
-  if (i > 0)
-    gtk_menu_insert (GTK_MENU (menu_shell), gtk_menu_item_new (),
-                                                            i + MENUFILE - 1);
 
   /* ja:メニュー表示 */
   set_menu_bar (NULL);
@@ -494,6 +499,7 @@ MMAID_COPYRIGHT_NOTICE"\n"
         {
           case FILE_KIND_ERROR:
           case FILE_KIND_DB:
+          case FILE_KIND_LIST:
             mmaid = file_open_edit (argv[i]);
             count++;
             break;
@@ -513,7 +519,8 @@ MMAID_COPYRIGHT_NOTICE"\n"
                       count++;
                     }
                   mm_list_append (MM_LIST (mmaid->list), id3tag);
-                  mmaid_change_edit (mmaid, TRUE);
+                  misc_mdi_set_edited (MISC_MDI (mdi),
+                    misc_mdi_get_page_from_data (MISC_MDI (mdi), mmaid), TRUE);
                 }
             }
         }
@@ -527,10 +534,14 @@ MMAID_COPYRIGHT_NOTICE"\n"
   profile = profile_open (NULL);
   if (profile)
     {
+      gint num;
+      GList *glist;
+
+      num = misc_history_get_num (MISC_HISTORY (history));
       profile_set_value (profile, "mmaid", "license",
             (gpointer)&license, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL);
       profile_set_value (profile, "mmaid", "history",
-            (gpointer)&history, sizeof (gint), PROFILE_VALUE_TYPE_INT);
+            (gpointer)&num, sizeof (gint), PROFILE_VALUE_TYPE_INT);
       profile_set_value (profile, "mmaid", "newfile",
             (gpointer)&newfile, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL);
       profile_set_value (profile, "mmaid", "second",
@@ -545,6 +556,14 @@ MMAID_COPYRIGHT_NOTICE"\n"
             (gpointer)&def_height, sizeof (gint), PROFILE_VALUE_TYPE_INT);
       profile_set_value (profile, "mmaid", "n_pos",
             (gpointer)&n_pos, sizeof (gint), PROFILE_VALUE_TYPE_INT);
+      profile_set_value (profile, "mmaid", "playlist",
+            (gpointer)&playlist, sizeof (gint), PROFILE_VALUE_TYPE_INT);
+      profile_set_value (profile, "mmaid", "cmd_play", (gpointer)cmd_play,
+                                (g_strlen (cmd_play) + 1) * sizeof (gchar),
+                                                    PROFILE_VALUE_TYPE_STRING);
+      profile_set_value (profile, "mmaid", "cmd_stop", (gpointer)cmd_stop,
+                                (g_strlen (cmd_stop) + 1) * sizeof (gchar),
+                                                    PROFILE_VALUE_TYPE_STRING);
       profile_set_value (profile, "mmaid", "charlist", (gpointer)charset_list,
                                 (g_strlen (charset_list) + 1) * sizeof (gchar),
                                                     PROFILE_VALUE_TYPE_STRING);
@@ -573,21 +592,23 @@ MMAID_COPYRIGHT_NOTICE"\n"
             }
         }
       /* ja:ファイルの履歴 */
-      for (i = 0; i < history && history_files[i]; i++)
+      glist = misc_history_get_file (MISC_HISTORY (history));
+      for (i = 0; i < g_list_length (glist); i++)
         {
           key = g_strdup_printf ("history%d", i);
-          utf8str = g_filename_to_utf8 (history_files[i], -1,
+          utf8str = g_filename_to_utf8 (g_list_nth_data (glist, i), -1,
                                                             NULL, NULL, NULL);
-          if (utf8str != NULL)
+          if (utf8str)
             {
               profile_set_value (profile, "mmaid", key, (gpointer)utf8str,
                                     (g_strlen (utf8str) + 1) * sizeof (gchar),
                                                     PROFILE_VALUE_TYPE_STRING);
               g_free (utf8str);
             }
-          g_free(key);
+          g_free (key);
         }
-      while (i < 10)
+      g_list_free (glist);
+      while (i <= MISC_HISTORY_NUM_MAX)
         {
           key = g_strdup_printf ("history%d", i++);
           profile_delete_key (profile, "mmaid", key);
@@ -623,10 +644,11 @@ MMAID_COPYRIGHT_NOTICE"\n"
   for (i = 0; i < genres; i++)
     for (j = 0; j < 256; j++)
       g_free (genre[i][j]);
+  g_free (cmd_play);
+  g_free (cmd_stop);
+  g_free (charset_list);
   g_free (open_path);
   g_free (save_path);
   g_free (clipboard_id3tag);
-  for (i = 0; i < history; i++)
-    g_free (history_files[i]);
   return 0;
 }
