@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,22 +33,17 @@ extern "C" {
 
 
 #define MMAID_VERSION_NOTICE "Melody maid "VERSION" ("MMAID_BUILD_ENVIRONMENT")"
-#define MMAID_COPYRIGHT_NOTICE "Copyright (C) 1998-2004 Kazuki IWAMOTO"
+#define MMAID_COPYRIGHT_NOTICE "Copyright (C) 1998-2005 Kazuki IWAMOTO"
 
 #define MENUFILE 10
 
 
 typedef struct MmaidWindow_Tag
 {
-  gboolean create;              /* ja:TRUE:新規作成,FALSE:既存ファイル */
-  gboolean edit;                /* ja:TRUE:編集済み,FALSE:未編集 */
   gchar *charset;               /* ja:キャラクターセット */
-  gchar *file;                  /* ja:ファイル名 */
-  gint same;                    /* ja:-1:重複なし,0?:番号 */
+  guint filekind;               /* ja:ファイルの種類 */
+  gint playlist;                /* ja:プレイリスト(0:相対,1:絶対,2:名前) */
   GtkWidget *list;              /* ja:描画領域 */
-  GtkWidget *label;             /* ja:タブのラベル */
-  GtkWidget *mark;              /* ja:タブの編集済みマーク */
-  GtkWidget *menu_item;         /* ja:メニューのウインドウの項目 */
 } MmaidWindow;
 typedef struct ID3Tag_Tag
 {
@@ -73,20 +68,22 @@ enum
 extern gboolean newfile;                /* ja:TRUE:新規ファイル,FALSE:なし */
 extern gboolean second;                 /* ja:TRUE:新規,FALSE:既存 */
 extern gboolean def_hold;               /* ja:TRUE:保持,FALSE:上書き */
-extern gchar *history_files[10];        /* ja:ファイルの履歴 */
 extern gchar *open_path;                /* ja:開くのパス */
 extern gchar *save_path;                /* ja:名前を付けて保存のパス */
 extern gchar *clipboard_id3tag;         /* ja:クリップボードのタグ */
 extern gchar *charset_list;             /* ja:キャラクターセットリスト */
+extern gchar *cmd_play;                 /* ja:演奏コマンド */
+extern gchar *cmd_stop;                 /* ja:停止コマンド */
 extern gchar *genre[256][256];          /* ja:ジャンル */
 extern gint genres;                     /* ja:ジャンルの数 */
 extern gint def_gnum;                   /* ja:デフォルトのジャンル */
 extern gint def_width, def_height;      /* ja:デフォルトのウインドウのサイズ */
-extern gint history;                    /* ja:ファイルの履歴 */
+extern gint playlist;                   /* ja:プレイリスト */
+extern GObject *history;                /* ja:ファイルの履歴 */
 extern GdkAtom atom_targets;            /* ja:ターゲット */
 extern GdkAtom atom_id3tag;             /* ja:ID3タグ */
 extern GtkWidget *window;               /* ja:ウインドウ */
-extern GtkWidget *notebook;             /* ja:ノートブック */
+extern GtkWidget *mdi;                  /* en:MDI */
 extern GtkPositionType n_pos;           /* ja:ノートブックの位置 */
 extern GtkItemFactory *ifactory_menu;   /* ja:メニュー */
 extern GtkItemFactory *ifactory_popup;  /* ja:ポップアップ */
