@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 #include <gdk/gdkkeysyms.h>
-#include "charlist.h"
 #include "reload.h"
+#include "misc/misc_charset.h"
 
 
 /******************************************************************************
@@ -27,7 +27,7 @@
 *                                                                             *
 ******************************************************************************/
 static gboolean reload_clicked_ok;
-static GtkWidget *dialog, *button0, *combo;
+static GtkWidget *dialog, *button0, *charset;
 
 
 /* ja:OKボタンの表示/非表示を判定 */
@@ -36,8 +36,7 @@ reload_dialog_ok_sensitive (GtkWidget *widget,
                             gpointer   user_data)
 {
   gtk_widget_set_sensitive (button0,
-                charlist_is_valid (gtk_entry_get_text
-                                    (GTK_ENTRY (GTK_COMBO (combo)->entry))));
+                            misc_charset_is_valid (MISC_CHARSET (charset)));
 }
 
 
@@ -47,8 +46,7 @@ reload_dialog_clicked (GtkWidget   *widget,
                        MmaidWindow *mmaid)
 {
   g_free (mmaid->charset);
-  mmaid->charset = g_strdup
-                (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry)));
+  mmaid->charset = misc_charset_get_charset (MISC_CHARSET (charset));
   gtk_widget_destroy (dialog);
   reload_clicked_ok = TRUE;
 }
@@ -83,9 +81,6 @@ gboolean
 reload_dialog (MmaidWindow *mmaid,
                const gchar *text)
 {
-  gchar **charset;
-  gint i;
-  GList *glist = NULL;
   GtkWidget *button1, *hbox, *vbox;
 
   /* ja:メインウインドウ */
@@ -112,30 +107,14 @@ reload_dialog (MmaidWindow *mmaid,
                         G_CALLBACK (gtk_widget_destroy), G_OBJECT (dialog));
   GTK_WIDGET_SET_FLAGS (button0, GTK_CAN_DEFAULT);
   GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
-  /* ja:コンボボックス */
-  glist = NULL;
-  charset = g_strsplit (charset_list, ",", G_MAXINT);
-  for (i = 0; charset[i]; i++)
-    if (!g_list_find_custom (glist, charset[i], (GCompareFunc)g_strcasecmp))
-      glist = g_list_insert_sorted (glist, g_strdup (charset[i]),
-                                                (GCompareFunc)g_strcasecmp);
-  g_strfreev (charset);
-  if (!g_list_find_custom (glist, mmaid->charset, (GCompareFunc)g_strcasecmp))
-    glist = g_list_insert_sorted (glist, g_strdup (mmaid->charset),
-                                                (GCompareFunc)g_strcasecmp);
-  combo = gtk_combo_new ();
-  gtk_combo_set_popdown_strings (GTK_COMBO (combo), glist);
-  gtk_combo_set_value_in_list (GTK_COMBO (combo), FALSE, FALSE);
-  gtk_combo_set_case_sensitive (GTK_COMBO (combo), TRUE);
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), mmaid->charset);
-  gtk_editable_select_region (GTK_EDITABLE (GTK_COMBO (combo)->entry), 0, -1);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo)->entry), "activate",
+  /* ja:文字符号化方式 */
+  charset = misc_charset_new ();
+  misc_charset_add_candidate (MISC_CHARSET (charset), charset_list);
+  misc_charset_set_charset (MISC_CHARSET (charset), mmaid->charset);
+  g_signal_connect (MISC_CHARSET (charset), "activate",
                                 G_CALLBACK (reload_dialog_activate), mmaid);
-  g_signal_connect (G_OBJECT (GTK_COMBO (combo)->entry), "changed",
+  g_signal_connect (MISC_CHARSET (charset), "edited",
                                 G_CALLBACK (reload_dialog_ok_sensitive), NULL);
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
-    g_free (g_list_nth_data (glist, i));
-  g_list_free (glist);
   /* ja:フレームとボックス */
   hbox = gtk_hbox_new (FALSE, SPACING);
   gtk_box_pack_end (GTK_BOX (hbox), button1, FALSE, FALSE, 0);
@@ -144,7 +123,7 @@ reload_dialog (MmaidWindow *mmaid,
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
   if (text)
     gtk_box_pack_start (GTK_BOX (vbox), gtk_label_new (text), FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), combo, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), charset, FALSE, FALSE, 0);
   gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_container_add (GTK_CONTAINER (dialog), vbox);
 
