@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "mm_list.h"
 #include "root.h"
 #include "misc/fileio.h"
+#include "misc/misc_mdi.h"
 
 
 /******************************************************************************
@@ -32,6 +33,7 @@ static gboolean unselect_all;
 static GtkWidget *dialog, *button1, *combo, *entry;
 
 
+#if ! GTK_CHECK_VERSION(2,4,0)
 /* ja:参照でOKボタンが押された */
 static void
 root_dialog_clicked_browse_ok (GtkWidget *widget,
@@ -44,6 +46,7 @@ root_dialog_clicked_browse_ok (GtkWidget *widget,
                 (GTK_FILE_SELECTION (dialog_browse)), -1, NULL, NULL, NULL);
   gtk_widget_destroy (dialog_browse);
 }
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
 
 
 /* ja:参照ボタンが押された */
@@ -51,10 +54,49 @@ static void
 root_dialog_clicked_browse (GtkWidget *widget,
                             gpointer   user_data)
 {
-  gchar *path = NULL;
-  const gchar *root;
+  gchar *root, *path;
   GtkWidget *dialog_browse;
 
+  root = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
+  if (root[0] == '\0' || root[g_strlen (root) - 1] != G_DIR_SEPARATOR)
+#if GTK_CHECK_VERSION(2,6,0)
+    root = gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo));
+#elif GTK_CHECK_VERSION(2,4,0)
+    {
+      GtkTreeIter iter;
+
+      if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter))
+        gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (combo)),
+                                                        &iter, 0, &root, -1);
+      else
+        root = NULL;
+    }
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+    root = g_strdup (gtk_entry_get_text
+                                    (GTK_ENTRY (GTK_COMBO (combo)->entry)));
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
+  path = root && root[0] != '\0'
+                                && root[g_strlen (root) - 1] == G_DIR_SEPARATOR
+                    ? g_filename_from_utf8 (root, -1, NULL, NULL, NULL) : NULL;
+  g_free (root);
+#if GTK_CHECK_VERSION(2,4,0)
+  dialog_browse = gtk_file_chooser_dialog_new (_("Browse"),
+                        GTK_WINDOW (window),
+                        GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+                        GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                        NULL);
+  if (path)
+    {
+      gtk_file_chooser_set_current_folder
+                                    (GTK_FILE_CHOOSER (dialog_browse), path);
+      g_free (path);
+    }
+  path = gtk_dialog_run (GTK_DIALOG (dialog_browse)) == GTK_RESPONSE_ACCEPT
+            ? gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog_browse))
+            : NULL;
+  gtk_widget_destroy (dialog_browse);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   dialog_browse = gtk_file_selection_new (_("Browse"));
   g_signal_connect (G_OBJECT (dialog_browse), "destroy",
                                             G_CALLBACK (gtk_main_quit), NULL);
@@ -64,17 +106,13 @@ root_dialog_clicked_browse (GtkWidget *widget,
                 (G_OBJECT (GTK_FILE_SELECTION (dialog_browse)->cancel_button),
        "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog_browse));
   g_object_set_data (G_OBJECT (dialog_browse), "user_data", &path);
-  root = gtk_entry_get_text (GTK_ENTRY (entry));
-  if (root[0] == '\0' || root[g_strlen (root) - 1] != G_DIR_SEPARATOR)
-    root = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry));
-  if (root[0] != '\0' && root[g_strlen (root) - 1] == G_DIR_SEPARATOR)
+  if (path)
     {
-      path = g_filename_to_utf8 (root, -1, NULL, NULL, NULL);
       gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog_browse),
                                                                         path);
       g_free (path);
+      path = NULL;
     }
-  path = NULL;
   gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (dialog_browse));
   gtk_widget_hide
     (gtk_widget_get_parent (GTK_FILE_SELECTION (dialog_browse)->file_list));
@@ -82,9 +120,14 @@ root_dialog_clicked_browse (GtkWidget *widget,
   gtk_widget_show (dialog_browse);
   gtk_grab_add (dialog_browse);
   gtk_main ();
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   if (path)
     {
-      gtk_entry_set_text (GTK_ENTRY (entry), path);
+      gchar *utf8str;
+
+      utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
+      gtk_entry_set_text (GTK_ENTRY (entry), utf8str);
+      g_free (utf8str);
       g_free (path);
     }
 }
@@ -96,11 +139,28 @@ root_dialog_clicked_ok (GtkWidget   *widget,
                         MmaidWindow *mmaid)
 {
   gchar *replace, *tmp;
+#if GTK_CHECK_VERSION(2,4,0)
+  gchar *find;
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   const gchar *find;
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   gint i, count, leng;
   const ID3Tag *id3tag;
 
-  find    = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry));
+
+#if GTK_CHECK_VERSION(2,6,0)
+  find = gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo));
+#elif GTK_CHECK_VERSION(2,4,0)
+    {
+      GtkTreeIter iter;
+
+      if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter))
+        gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (combo)),
+                                                        &iter, 0, &find, -1);
+    }
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+  find = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry));
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   replace = g_filename_from_utf8 (gtk_entry_get_text (GTK_ENTRY (entry)), -1,
                                                             NULL, NULL, NULL);
   tmp = fileio_get_full_path (replace);
@@ -117,8 +177,12 @@ root_dialog_clicked_ok (GtkWidget   *widget,
           mm_list_change (MM_LIST (mmaid->list), i, MM_LIST_FILE,
                         g_strconcat (replace, id3tag->file + leng, NULL), -1);
       }
+#if GTK_CHECK_VERSION(2,4,0)
+  g_free (find);
+#endif /* GTK_CHECK_VERSION(2,4,0) */
   g_free (replace);
-  mmaid_change_edit (mmaid, TRUE);
+  misc_mdi_set_edited (MISC_MDI (mdi),
+                    misc_mdi_get_page_from_data (MISC_MDI (mdi), mmaid), TRUE);
   gtk_widget_destroy (dialog);
 }
 
@@ -140,11 +204,27 @@ root_dialog_changed (GtkWidget   *widget,
 {
   gboolean result = TRUE;
   gchar *file, *replace, *tmp;
+#if GTK_CHECK_VERSION(2,4,0)
+  gchar *find;
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   const gchar *find;
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   gint i, count, leng;
   const ID3Tag *id3tag;
 
-  find    = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry));
+#if GTK_CHECK_VERSION(2,6,0)
+  find = gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo));
+#elif GTK_CHECK_VERSION(2,4,0)
+    {
+      GtkTreeIter iter;
+
+      if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter))
+        gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (combo)),
+                                                        &iter, 0, &find, -1);
+    }
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+  find = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry));
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   replace = g_filename_from_utf8 (gtk_entry_get_text (GTK_ENTRY (entry)), -1,
                                                             NULL, NULL, NULL);
   tmp = fileio_get_full_path (replace);
@@ -164,6 +244,9 @@ root_dialog_changed (GtkWidget   *widget,
             g_free (file);
           }
       }
+#if GTK_CHECK_VERSION(2,4,0)
+  g_free (find);
+#endif /* GTK_CHECK_VERSION(2,4,0) */
   g_free (replace);
   gtk_widget_set_sensitive (button1, result);
 }
@@ -186,9 +269,9 @@ root_dialog_key_press (GtkWidget   *widget,
 void
 root_dialog (MmaidWindow *mmaid)
 {
-  gint i, j, count;
+  gint i, count;
   GList *glist = NULL;
-  GtkWidget *button0, *button2, *hbox0, *hbox1, *vbox;
+  GtkWidget *button0, *button2, *label0, *label1, *hbox0, *hbox1, *vbox;
 
   /* ja:メインウインドウ */
   dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -223,6 +306,7 @@ root_dialog (MmaidWindow *mmaid)
     if (unselect_all || mm_list_is_selected (MM_LIST (mmaid->list), i))
       {
         gchar **dir, *path = NULL;
+        gint j;
         const ID3Tag *id3tag;
 
         id3tag = mm_list_get_nth (MM_LIST (mmaid->list), i);
@@ -261,14 +345,23 @@ root_dialog (MmaidWindow *mmaid)
         g_strfreev (dir);
         g_free (path);
       }
+#if GTK_CHECK_VERSION(2,4,0)
+  combo = gtk_combo_box_new_text ();
+  for (i = g_list_length (glist) - 1; i >= 0; i--)
+    gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo),
+                                                g_list_nth_data (glist, i));
+  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
+  g_signal_connect (G_OBJECT (combo), "changed",
+                                    G_CALLBACK (root_dialog_changed), mmaid);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   combo = gtk_combo_new ();
   gtk_combo_set_popdown_strings (GTK_COMBO (combo), glist);
   gtk_combo_set_value_in_list (GTK_COMBO (combo), FALSE, FALSE);
-#ifdef G_OS_WIN32
+# ifdef G_OS_WIN32
   gtk_combo_set_case_sensitive (GTK_COMBO (combo), FALSE);
-#else /* not G_OS_WIN32 */
+# else /* not G_OS_WIN32 */
   gtk_combo_set_case_sensitive (GTK_COMBO (combo), TRUE);
-#endif /* not G_OS_WIN32 */
+# endif /* not G_OS_WIN32 */
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry),
                                                 g_list_nth_data (glist, 0));
   gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (combo)->entry), FALSE);
@@ -276,6 +369,7 @@ root_dialog (MmaidWindow *mmaid)
                                     G_CALLBACK (root_dialog_activate), mmaid);
   g_signal_connect (G_OBJECT (GTK_COMBO (combo)->entry), "changed",
                                     G_CALLBACK (root_dialog_changed), mmaid);
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   for (i = g_list_length (glist) - 1; i >= 0; i--)
     g_free (g_list_nth_data (glist, i));
   g_list_free (glist);
@@ -285,20 +379,27 @@ root_dialog (MmaidWindow *mmaid)
                                     G_CALLBACK (root_dialog_changed), mmaid);
   g_signal_connect (G_OBJECT (entry), "activate",
                                     G_CALLBACK (root_dialog_activate), mmaid);
+  /* ja:ラベル */
+  label0 = gtk_label_new_with_mnemonic (_("_Find Path"));
+  label1 = gtk_label_new_with_mnemonic (_("_Replace Path"));
+#if GTK_CHECK_VERSION(2,4,0)
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label0), combo);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label0), GTK_COMBO (combo)->entry);
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label1), entry);
   /* ja:フレームとボックス */
   vbox = gtk_vbox_new (FALSE, SPACING);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
   hbox0 = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox0),
-                        gtk_label_new (_("Find Path")), FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox0), label0, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox0), combo, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox0, FALSE, FALSE, 0);
   hbox0 = gtk_hbox_new (FALSE, SPACING);
   hbox1 = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox1),
-                        gtk_label_new (_("Replace Path")), FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox1), entry, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox0), hbox1, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox0), hbox1, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox0), button0, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox0, FALSE, FALSE, 0);
   hbox0 = gtk_hbox_new (FALSE, SPACING);
