@@ -28,6 +28,17 @@ extern "C" {
 #endif /* __cplusplus */
 
 
+/******************************************************************************
+*                                                                             *
+* ja:シグナル/イベント関数群(マウス)                                          *
+*                                                                             *
+******************************************************************************/
+gboolean
+signal_button_press (GtkWidget      *widget,
+                     GdkEventButton *event,
+                     MmaidWindow    *mmaid);
+
+
 /******************************************************************************
 *                                                                             *
 * ja:シグナル/イベント関数群(フレーム)                                        *
