@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,35 +34,25 @@ extern "C" {
 *                                                                             *
 ******************************************************************************/
 void
-signal_activate_menu_window (GtkWidget *widget,
-                             GtkWidget *child);
-
-
-void
-signal_activate_menu_history (GtkWidget *widget,
-                              gchar     *file);
-
-
-void
-signal_destroy_menu_history (GtkWidget *widget,
-                             gchar     *file);
+signal_activate_menu_history (GtkWidget   *widget,
+                              const gchar *file);
 
 
 /******************************************************************************
 *                                                                             *
-* ja:シグナル/イベント関数群(ノート)                                          *
+* ja:シグナル/イベント関数群(MDI)                                             *
 *                                                                             *
 ******************************************************************************/
 void
-signal_clicked_button (GtkWidget *widget,
-                       GtkWidget *child);
+signal_close_window (GtkWidget   *widget,
+                     guint        page_num,
+                     MmaidWindow *mmaid);
 
 
 void
-signal_switch_page (GtkNotebook     *notebook,
-                    GtkNotebookPage *page,
-                    gint             page_num,
-                    gpointer         user_data);
+signal_switch_window (GtkWidget   *widget,
+                      guint        page_num,
+                      MmaidWindow *mmaid);
 
 
 /******************************************************************************
@@ -118,11 +108,6 @@ signal_delete (GtkWidget *widget,
                gpointer   user_data);
 
 
-void
-signal_destroy_window (GtkWidget *widget,
-                       gpointer user_data);
-
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
