@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -309,29 +309,29 @@ static gchar *tb_edit_xpm[] = {
 
 /* XPM */
 static gchar *tb_search_xpm[] = {
-"16 16 5 1",
-"   c None",
-".  c #000000",
-"=  c #C0C0C0",
-",  c #0000FF",
-"!  c #FFFFFF",
-"  .........     ",
-"  .!!!!!!!.     ",
-"  .!!!!!!!.     ",
-"  .!!!!!!!.     ",
-"  .!!!!!!!.     ",
-"  .!!!!!!!.     ",
-"  .!!!!!!...    ",
-"  .!!!!!.=. .   ",
-"  .!!!!.==.!!.  ",
-"  .!!!!.==. !.  ",
-"  .........  .  ",
-"       ,.   .   ",
-"      ,,,...    ",
-"     ,,,        ",
-"    ,,,         ",
-"     ,          "};
-
+"16 16 6 1",
+" 	c None",
+".	c #000000",
+"*	c #C0C0C0",
+";	c #FFFF00",
+">	c #0000FF",
+")	c #FFFFFF",
+"                ",
+"  ....          ",
+" .;);).....     ",
+" .););););.     ",
+" .;);).....     ",
+" .....);)...    ",
+" .;););).*.).   ",
+" .););).*;.)).  ",
+" .;););.;*.)).  ",
+" ..........)).  ",
+"       >.))).   ",
+"      >>>...    ",
+"     >>>        ",
+"    >>>         ",
+"     >          ",
+"                "};
 
 /* XPM */
 static gchar *tb_up_xpm[] = {
