@@ -23,13 +23,24 @@
 #include "mmaid.h"
 
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
+
 /******************************************************************************
 *                                                                             *
 * ja:バージョン情報関数群                                                     *
 *                                                                             *
 ******************************************************************************/
+/*  ja:バージョン表示                                                       */
 void
 version_dialog (void);
 
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
+
 #endif
